#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

#include <gtk/gtk.h>
#include <gdk/gdkx.h>
#include <gdk/gdkkeysyms.h>

#ifdef HAVE_STDLIB_H
# include <stdlib.h>
#endif

#include "callbacks.h"
#include "interface.h"
#include "support.h"
#include "penguzilla.h"
#include <string.h>
#include <stdio.h>

#ifdef HAVE_UNISTD_H
# include <unistd.h>
# include <sys/types.h>
# include <sys/wait.h>
# include <signal.h>
pid_t kbd_pid = 0;
#endif

#define PENGUDRIVE 1
//#undef PENGUDRIVE
    
extern button_group tastatur;
extern button_group toolbar;
extern keyboard1;
extern char *device_file_name;
extern int device_width;
extern gboolean toolbar_all;
extern gboolean toolbar_load;
extern gboolean URI_scrolled;
extern gboolean nogprs;
extern GtkWidget *windowTop, *combo_list, *URIlabel [15];
extern GList *URI_history;
extern char *settings [];
extern gint URI_history_pos;
extern FILE *fhan;

extern void save_rc (char *rc_filename, GtkWidget *widget);

gboolean link_status = FALSE;
gboolean keyboard_open = FALSE;

#define GLADE_HOOKUP_OBJECT(component,widget,name) \
	g_object_set_data_full (G_OBJECT (component), name, \
					gtk_widget_ref (widget), (GDestroyNotify) gtk_widget_unref)

#ifdef PENGUDRIVE
#include <stdio.h>

extern char *device_file_name;
extern int device_width;

/**
 * activate_pengudrive
 * 
 * @i:		overall number of clicks
 * @pos:	initial position
 */
void activate_pengudrive (gchar *f, gint i, gint pos) 
{
//	static gint oi = -1, opos = -1;
	
//	if ((i == oi) && (opos == pos)) return;
//	oi = i;
//	opos = pos;
	
	if (f) g_print ("%s: %s (%d/%d)\n", __FUNCTION__, f, i, pos);
	else g_print ("%s: (NULL) (%d/%d)\n", __FUNCTION__, i, pos);
	
	if (!fhan) 
	{
		g_message ("could not open NAMED_PIPE\n");
		return;
	}
		
	fprintf (fhan, "S%d,%d,%d;\n", i, device_width, pos);
	fflush (fhan);
	
	g_print ("%s: effect sended\n", __FUNCTION__);
	return;
}

void disable_pengudrive (gchar *f) 
{
	g_print ("%s: %s\n", __FUNCTION__, f);
	
	if (!fhan) 
	{
		g_message ("could not open NAMED_PIPE\n");
		return;
	}
	
	fprintf (fhan, "R;\n");
	fflush (fhan);
	g_print ("%s: effect sended\n", __FUNCTION__);
	return;
}
#else
void activate_pengudrive (gchar *f, gint i, gint pos) 
{
	g_print ("PENGUDRIVE activated: %s (%d/%d)\n", 
		 __FUNCTION__, f, i, pos);
}
void disable_pengudrive (gchar *f) 
{
	g_print ("PENGUDRIVE deactivated: %s\n", __FUNCTION__, f);
}
#endif


gint x_scroll = 0, y_scroll = 0, vbalk = 0, loadakt = 0;

void show_tastatur (GtkWidget *wid);

void send_keycode (GtkWidget *wid, GtkDirectionType direction)
{
	XKeyEvent ev;
	Display *display = GDK_DISPLAY ();
	
	ev.type = KeyPress;
	ev.display = GDK_DISPLAY ();
	ev.window = GDK_WINDOW_XWINDOW (wid->window);
	ev.root = GDK_ROOT_WINDOW ();
	ev.time = GDK_CURRENT_TIME;
	ev.x = 0;
	ev.y = 0;
	ev.x_root = 0;
	ev.y_root = 0;
	ev.state = 0;
	ev.same_screen = TRUE;
	
	switch (direction)
	{
		
	case GTK_DIR_UP:
		ev.keycode =  XKeysymToKeycode(GDK_DISPLAY(), GDK_Up);
		break;
	case GTK_DIR_DOWN:
		ev.keycode =  XKeysymToKeycode(GDK_DISPLAY(), GDK_Down);
		break;
	case GTK_DIR_LEFT:
		ev.keycode =  XKeysymToKeycode(GDK_DISPLAY(), GDK_Left);
		break;
	case GTK_DIR_RIGHT:
		ev.keycode =  XKeysymToKeycode(GDK_DISPLAY(), GDK_Right);
		break;
	case GTK_DIR_TAB_FORWARD:
		ev.keycode =  XKeysymToKeycode(GDK_DISPLAY(), GDK_Tab);
		break;
	case GTK_DIR_TAB_BACKWARD:
		ev.keycode =  XKeysymToKeycode(GDK_DISPLAY(), GDK_Tab);
		ev.state = ShiftMask;
		break;
	}
	
	XSendEvent (gdk_display,
		    GDK_WINDOW_XWINDOW (wid->window),
		    False, NoEventMask, &ev);
}


void
    gtk_set_mouse_position (GtkWidget *wid, int x, int y)
{
	Display *display = GDK_DISPLAY ();
	XWarpPointer (display, None,
		      GDK_WINDOW_XWINDOW (wid->window),
		      0, 0, 0, 0, x, y);
}

void chg_pixmap (GtkWidget *button, const char *img_name, const char *pic_name)
{	
	GtkWidget *image;
	
	g_print ("%s %s %s\n", __FUNCTION__, img_name, pic_name);
	
	image = lookup_widget (GTK_WIDGET (button), img_name);
	gtk_widget_hide (image);	
	gtk_widget_destroy (image);
	
	image = create_pixmap (windowTop, pic_name);
	gtk_widget_set_name (image, img_name);
	gtk_widget_show (image);
	gtk_container_add (GTK_CONTAINER (button), image);
	GLADE_HOOKUP_OBJECT (windowTop, image, img_name);
}

void add_char (AtkObject *aobj, gchar *s, guint l)
{
	guint pos = 0;
	gchar *text;
	
	if (ATK_IS_EDITABLE_TEXT (aobj))
	{
		pos = atk_text_get_caret_offset (ATK_TEXT (aobj));
		g_print ("pos before: %d\n", pos);
		atk_editable_text_insert_text (ATK_EDITABLE_TEXT (aobj), s, l, 
				       &pos);
		g_print ("pos after : %d\n", pos);
		atk_text_set_caret_offset (ATK_TEXT (aobj), pos + l);
		g_signal_emit_by_name (ATK_TEXT (aobj), 
				       "text-changed");
	} 
	else g_print ("%s\n", atk_object_get_name (aobj)); 
}

/* {{{ Sensitive Borders for Scrolling */

gint scroll_up (gpointer widget)
{
	GtkAdjustment *vadjustment;
	gfloat vvalue;
    
	g_print("%s: starting\n",__FUNCTION__);
	vadjustment = gtk_viewport_get_vadjustment (GTK_VIEWPORT(lookup_widget (GTK_WIDGET(widget),"viewport1")));

	g_print("%s: value: %f\tlower: %f\tupper: %f\n", __FUNCTION__, 
		vadjustment->value, vadjustment->lower, vadjustment->upper);

	vvalue = vadjustment->value - 5;
 	if (vvalue < 0) vvalue = 0;
	gtk_adjustment_set_value(vadjustment, vvalue);
	gtk_viewport_set_vadjustment (GTK_VIEWPORT(lookup_widget (GTK_WIDGET(widget),"viewport1")), vadjustment);
	
	g_print("%s: finished\n",__FUNCTION__);

	return TRUE;
}

gint scroll_down (gpointer widget)
{
    	GtkAdjustment *vadjustment;
	gfloat vvalue;
	
	g_print("%s: starting\n",__FUNCTION__);
	vadjustment = gtk_viewport_get_vadjustment (GTK_VIEWPORT(lookup_widget (GTK_WIDGET(widget),"viewport1")));

	g_print("%s: value: %f\tlower: %f\tupper: %f\n", __FUNCTION__, 
		vadjustment->value, vadjustment->lower, vadjustment->upper);

	vvalue = vadjustment->value + 5;
 	if (vvalue > vadjustment->upper - vadjustment->page_size) 
		vvalue = vadjustment->upper - vadjustment->page_size;
	gtk_adjustment_set_value(vadjustment, vvalue);
	gtk_viewport_set_vadjustment (GTK_VIEWPORT(lookup_widget (GTK_WIDGET(widget),"viewport1")), vadjustment);

	g_print("%s: finished\n",__FUNCTION__);
	return TRUE;
}

gint scroll_left (gpointer widget)
{
    	GtkAdjustment *hadjustment;
	gfloat hvalue;
	
	g_print("%s: starting\n",__FUNCTION__);
	hadjustment = gtk_viewport_get_hadjustment (GTK_VIEWPORT(lookup_widget (GTK_WIDGET(widget),"viewport1")));

	g_print("%s: value: %f\tlower: %f\tupper: %f\n", __FUNCTION__, 
		hadjustment->value, hadjustment->lower, hadjustment->upper);

	hvalue = hadjustment->value - 5;
 	if (hvalue < 0) hvalue = 0;
	gtk_adjustment_set_value(hadjustment, hvalue);
	gtk_viewport_set_hadjustment (GTK_VIEWPORT(lookup_widget (GTK_WIDGET(widget),"viewport1")), hadjustment);
	
	g_print("%s: finished\n",__FUNCTION__);

	return TRUE;
}

gint scroll_right (gpointer widget)
{
    	GtkAdjustment *hadjustment;
	gfloat hvalue;
	
	g_print("%s: starting\n",__FUNCTION__);
	hadjustment = gtk_viewport_get_hadjustment (GTK_VIEWPORT(lookup_widget (GTK_WIDGET(widget),"viewport1")));

 	g_print("%s: value: %f\tlower: %f\tupper: %f\n", __FUNCTION__, 
		hadjustment->value, hadjustment->lower, hadjustment->upper);
    
	hvalue = hadjustment->value + 5;
 	if (hvalue > (hadjustment->upper - hadjustment->page_size)) 
		hvalue = hadjustment->upper - hadjustment->page_size;
	gtk_adjustment_set_value(hadjustment, hvalue);
	gtk_viewport_set_hadjustment (GTK_VIEWPORT(lookup_widget (GTK_WIDGET(widget),"viewport1")), hadjustment);

	g_print("%s: finished\n",__FUNCTION__);

	return TRUE;
}

gboolean
on_eventbox3_button_press_event        (GtkWidget       *widget,
                                        GdkEventButton  *event,
                                        gpointer         user_data)
{
	gtk_widget_show (lookup_widget (widget, "hbox4"));
	gtk_widget_hide (lookup_widget (windowTop,
					"label_tastatur_dummy"));
	gtk_widget_hide (lookup_widget (windowTop,
					"label_tastatur1_dummy"));
	gtk_set_mouse_position (windowTop, 300, 2);
	return FALSE;
}

gboolean
on_bordertop_enter_notify_event        (GtkWidget       *widget,
                                        GdkEventCrossing *event,
                                        gpointer         xuser_data)
{
	g_print ("%s: starting\n", __FUNCTION__);
	y_scroll = gtk_timeout_add (100, scroll_up, widget);
	g_print ("%s: finished\n", __FUNCTION__);
	return FALSE;
}


gboolean
on_bordertop_leave_notify_event        (GtkWidget       *widget,
                                        GdkEventCrossing *event,
                                        gpointer         user_data)
{
	g_print ("%s: starting\n", __FUNCTION__);
	gtk_timeout_remove (y_scroll);
	g_print ("%s: finished\n", __FUNCTION__);
	return FALSE;
}

gboolean
on_borderbottom_enter_notify_event     (GtkWidget       *widget,
                                        GdkEventCrossing *event,
                                        gpointer         user_data)
{
	g_print ("%s: starting\n", __FUNCTION__);
	y_scroll = gtk_timeout_add (100, scroll_down, widget);
	g_print ("%s: finished\n", __FUNCTION__);
	return FALSE;
}


gboolean
on_borderbottom_leave_notify_event     (GtkWidget       *widget,
                                        GdkEventCrossing *event,
                                        gpointer         user_data)
{
	g_print ("%s: starting\n", __FUNCTION__);
	gtk_timeout_remove (y_scroll);
	g_print ("%s: finished\n", __FUNCTION__);
	return FALSE;
}


gboolean
on_borderleft_enter_notify_event      (GtkWidget       *widget,
                                        GdkEventCrossing *event,
                                        gpointer         user_data)
{
	g_print ("%s: starting\n", __FUNCTION__);
	x_scroll = gtk_timeout_add (100, scroll_left, widget);
	g_print ("%s: finished\n", __FUNCTION__);
	return FALSE;
}


gboolean
on_borderleft_leave_notify_event      (GtkWidget       *widget,
                                        GdkEventCrossing *event,
                                        gpointer         user_data)
{
	g_print ("%s: starting\n",__FUNCTION__);
	gtk_timeout_remove (x_scroll);
	g_print ("%s: finished\n", __FUNCTION__);
	return FALSE;
}


gboolean
on_borderright_enter_notify_event     (GtkWidget       *widget,
				       GdkEventCrossing *event,
				       gpointer         user_data)
{
	g_print ("%s: starting\n", __FUNCTION__);
	x_scroll = gtk_timeout_add (100, scroll_right, widget);
	g_print ("%s: finished\n", __FUNCTION__);
	return FALSE;
}


gboolean
on_borderright_leave_notify_event     (GtkWidget       *widget,
                                        GdkEventCrossing *event,
                                        gpointer         user_data)
{
	g_print ("%s: starting\n", __FUNCTION__);
	gtk_timeout_remove (x_scroll);
	g_print ("%s: finished\n", __FUNCTION__);
	return FALSE;
}

/* }}} */

/* {{{ Callbacks */
gboolean knka (gpointer data)
{
	if (!GTK_WIDGET_VISIBLE (lookup_widget (windowTop, "progressbar1")))
	    return FALSE;
	
	progress_change_cb (lookup_widget (windowTop, "gecko"),
			    0, 0, NULL);
	
	return TRUE;
}

void
load_started_cb (GtkMozEmbed *embed)
{
	g_print("load started\n");
	toolbar_load = TRUE;
	URI_scrolled = FALSE;
	gtk_widget_show (lookup_widget (GTK_WIDGET (embed), 
					"progressbar1"));
	g_timeout_add (200, knka, windowTop);
}

void
load_finished_cb (GtkMozEmbed *embed)
{
	g_print("load finished\n");
	toolbar_load = FALSE;
	gtk_widget_hide (lookup_widget (GTK_WIDGET (embed), 
					"progressbar1"));
	location_changed_cb (embed, NULL);
}

void 
progress_change_cb (GtkMozEmbed	*embed, 
                    gint 	cur,
                    gint 	max,
                    gpointer	user_data)
{
	g_print ("progress_change_cb cur %d max %d\n", cur, max);
	
	if (max < 1) gtk_progress_bar_pulse (GTK_PROGRESS_BAR (lookup_widget (GTK_WIDGET (embed), 
									      "progressbar1")));
	else gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (lookup_widget (GTK_WIDGET (embed), 
									     "progressbar1")),
					    (gdouble) cur/ (gdouble) max);
}

gint URIcmp (gconstpointer a,
	     gconstpointer b)
{
	char *ap, *bp;
	int i;
	
	ap = strstr ((char *) a, "//");
	
	if (ap) ap += 2;
	else ap = (char *) a;
	
	bp = strstr ((char *) b, "//");
	
	if (bp) bp += 2;
	else bp = (char *) b;
	
	for (i = strlen (ap); i > 0; i--) 
	{
		switch (ap [i])
		{
		case '/': 
		case '\t':
		case ' ': ap [i] = 0; break;
		default: i = 0;
		}
	}
	
	for (i = strlen (bp); i > 0; i--) 
	{
		switch (bp [i])
		{
		case 0:
		case '/': 
		case '\t':
		case ' ': bp [i] = 0; break;
		default: i = 0;
		}
	}
		
	g_print ("%s:\nap = %s\nbp = %s\n", __FUNCTION__, ap, bp);
	
	return strcmp (ap, bp);
}

gboolean
location_changed_cb (GtkMozEmbed *embed, gpointer user_data)
{
	GtkWidget	*combo_entry_URI_entry;
	gchar 		*newLocation, *hist;
	int   		newPosition = 0;
	GtkAdjustment	*hadjustment, *vadjustment;
	
	g_print ("location_changed_cb\n");
	newLocation = gtk_moz_embed_get_location (embed);
	if (newLocation)
	{
		gtk_entry_set_text (lookup_widget (GTK_WIDGET (embed), 
						   "combo_entry_URI_entry"), 
				    newLocation);
		/* add to history */
		
		if (!g_list_find_custom (URI_history, newLocation, URIcmp))
		{
			hist = calloc (strlen (newLocation) + 1, 
				       sizeof (gchar));
			if (hist)
			{
				strcpy (hist, newLocation);
				
				URI_history = g_list_insert (URI_history, 
							     hist, 1);
				
				while (g_list_length (URI_history) > 10)
				{
					g_list_delete_link (URI_history,
							    g_list_last (URI_history));
				}
			}
		}
				
		vadjustment = gtk_viewport_get_vadjustment (GTK_VIEWPORT (lookup_widget (GTK_WIDGET (embed),
											 "viewport1")));
		hadjustment = gtk_viewport_get_hadjustment (GTK_VIEWPORT (lookup_widget (GTK_WIDGET (embed),
											 "viewport1")));

		gtk_adjustment_set_value (vadjustment, 0);
		gtk_viewport_set_vadjustment (GTK_VIEWPORT (lookup_widget (GTK_WIDGET (embed),
									  "viewport1")), 
					      vadjustment);
		gtk_adjustment_set_value (hadjustment, 0);
		gtk_viewport_set_hadjustment (GTK_VIEWPORT (lookup_widget (GTK_WIDGET (embed),
									  "viewport1")), 
					      hadjustment);
		return FALSE;
	}
	return TRUE;
}

gint
    dom_mouse_click_cb   (GtkMozEmbed *embed, gpointer user_data)
{
	AtkObject 	*aobj, *aobj1;
	AtkRole		arole, arole1;
	gint 		pos;
	gchar		*name, *name1;
	
	aobj1 = atk_get_focus_object ();

	aobj = atk_gobject_accessible_for_object (G_OBJECT (embed));
	arole = atk_object_get_role (aobj);
	name = atk_object_get_name (aobj);
	arole1 = atk_object_get_role (aobj1);
	name1 = atk_object_get_name (aobj1);
	
	g_print ("DOM: click on %s (%p)\n", (name) ? name : "(NULL)" , aobj);
	g_print ("DOM: %s\n", atk_role_get_name (arole));
	g_print ("children: %d\n",
		 atk_object_get_n_accessible_children (aobj));
	
	g_print ("DOM: focus on %s (%p)\n", (name) ? name : "(NULL)" , aobj1);
	g_print ("DOM: %s\n", atk_role_get_name (arole1));
	g_print ("children: %d\n",  
		 atk_object_get_n_accessible_children (aobj1));
	
//	atk_focus_tracker_notify (aobj);
	return 0;
}


gint
        dom_mouse_dbl_click_cb   (GtkMozEmbed *embed, gpointer user_data)
{
//	show_tastatur (embed);
}

GtkWidget*
    gtk_moz_embed_new_wrap (gchar *widget_name, gchar *string1, gchar *string2,
			    gint int1, gint int2)
{
	return gtk_moz_embed_new ();
}


/* }}} */

/* {{{ commented out */
#if 0

void
location_changed_cb (GtkMozEmbed *embed, TestGtkBrowser *browser){
  char *newLocation;
  int   newPosition = 0;
  g_print("location_changed_cb\n");
  newLocation = gtk_moz_embed_get_location(embed);
  if (newLocation)
  {
    gtk_editable_delete_text(GTK_EDITABLE(browser->urlEntry), 0, -1);
    gtk_editable_insert_text(GTK_EDITABLE(browser->urlEntry),
			     newLocation, strlen(newLocation), &newPosition);
    g_free(newLocation);
  }
  else
    g_print("failed to get location!\n");
  // always make sure to clear the tempMessage.  it might have been
  // set from the link before a click and we wouldn't have gotten the
  // callback to unset it.
  update_temp_message(browser, 0);
  // update the nav buttons on a location change
  update_nav_buttons(browser);
}

void
load_started_cb     (GtkMozEmbed *embed, TestGtkBrowser *browser)
{
  g_print("load_started_cb\n");
  gtk_widget_set_sensitive(browser->stopButton, TRUE);
  gtk_widget_set_sensitive(browser->reloadButton, FALSE);
  browser->loadPercent = 0;
  browser->bytesLoaded = 0;
  browser->maxBytesLoaded = 0;
  update_status_bar_text(browser);
}

void
load_finished_cb    (GtkMozEmbed *embed, TestGtkBrowser *browser)

gint dom_mouse_dbl_click_cb (GtkMozEmbed *embed, nsIDOMMouseEvent *event,
			     TestGtkBrowser *browser)
{
  g_print("dom_mouse_dbl_click_cb\n");
  return NS_OK;
}

gint dom_mouse_over_cb    (GtkMozEmbed *embed, nsIDOMMouseEvent *event,
			   TestGtkBrowser *browser)
{
  g_print("dom_mouse_over_cb\n");
  return NS_OK;
}

gint dom_mouse_out_cb     (GtkMozEmbed *embed, nsIDOMMouseEvent *event,
			   TestGtkBrowser *browser)
{
  g_print("dom_mouse_out_cb\n");
  return NS_OK;
}
// utility functions

void
update_status_bar_text(TestGtkBrowser *browser)
{
  gchar message[256];
  
  gtk_statusbar_pop(GTK_STATUSBAR(browser->statusBar), 1);
  if (browser->tempMessage)
    gtk_statusbar_push(GTK_STATUSBAR(browser->statusBar), 1, browser->tempMessage);
  else
  {
    if (browser->loadPercent)
    {
      g_snprintf(message, 255, "%s (%d%% complete, %d bytes of %d loaded)", browser->statusMessage, browser->loadPercent, browser->bytesLoaded, browser->maxBytesLoaded);
    }
    else if (browser->bytesLoaded)
    {
      g_snprintf(message, 255, "%s (%d bytes loaded)", browser->statusMessage, browser->bytesLoaded);
    }
    else if (browser->statusMessage == NULL)
    {
      g_snprintf(message, 255, " ");
    }
    else
    {
      g_snprintf(message, 255, "%s", browser->statusMessage);
    }
    gtk_statusbar_push(GTK_STATUSBAR(browser->statusBar), 1, message);
  }
}

void
update_temp_message(TestGtkBrowser *browser, const char *message)
{
  if (browser->tempMessage)
    g_free(browser->tempMessage);
  if (message)
    browser->tempMessage = g_strdup(message);
  else
    browser->tempMessage = 0;
  // now that we've updated the temp message, redraw the status bar
  update_status_bar_text(browser);
}


void
update_nav_buttons      (TestGtkBrowser *browser)
{
	gboolean can_go_back;
	gboolean can_go_forward;
	can_go_back = gtk_moz_embed_can_go_back (GTK_MOZ_EMBED (browser->mozEmbed));
	can_go_forward = gtk_moz_embed_can_go_forward (GTK_MOZ_EMBED (browser->mozEmbed));
	if (can_go_back)
	    gtk_widget_set_sensitive (browser->backButton, TRUE);
	else
	    gtk_widget_set_sensitive (browser->backButton, FALSE);
	if (can_go_forward)
	    gtk_widget_set_sensitive (browser->forwardButton, TRUE);
	else
	    gtk_widget_set_sensitive (browser->forwardButton, FALSE);
 }

#endif

/* }}} */

/* {{{ Toplevel Window */

gboolean
on_windowTop_destroy_event             (GtkWidget       *widget,
                                        GdkEvent        *event,
                                        gpointer         user_data)
{
	gtk_main_quit ();
	return FALSE;
}

void
on_windowTop_show                      (GtkWidget       *widget,
                                        gpointer         user_data)
{
	gtk_signal_connect (GTK_OBJECT (lookup_widget(widget, "gecko")),
			   "net_start",
			   GTK_SIGNAL_FUNC (load_started_cb),
			   lookup_widget(widget, "gecko"));
	
	gtk_signal_connect (GTK_OBJECT (lookup_widget(widget, "gecko")),
			   "net_stop",
			   GTK_SIGNAL_FUNC (load_finished_cb),
			   lookup_widget (widget, "gecko"));
	gtk_signal_connect (GTK_OBJECT (lookup_widget(widget, "gecko")),
			   "progress",
			   GTK_SIGNAL_FUNC (progress_change_cb),
			   lookup_widget (widget, "gecko"));
	gtk_signal_connect (GTK_OBJECT (lookup_widget (widget, "gecko")),
			   "open_uri",
			    GTK_SIGNAL_FUNC (location_changed_cb),
			    lookup_widget (widget, "gecko"));
	gtk_signal_connect (GTK_OBJECT (lookup_widget (widget, "gecko")),
			    "dom_mouse_click",
			    GTK_SIGNAL_FUNC (dom_mouse_click_cb),
			    lookup_widget (widget, "gecko"));
	gtk_signal_connect (GTK_OBJECT (lookup_widget (widget, "gecko")),
			    "dom_mouse_dbl_click",
			    GTK_SIGNAL_FUNC (dom_mouse_dbl_click_cb),
			    lookup_widget (widget, "gecko"));
}

/* }}} */

/* {{{ Generic Keyboard 0 Events */

gboolean fixate_mouse_tast (gpointer data)
{
	gtk_set_mouse_position (GTK_WIDGET (data), 600, 210);
	return tastatur.fix_mouse;
}

void show_tastatur (GtkWidget *wid)
{
	g_print ("%s\n", __FUNCTION__);
	
//	disable_pengudrive (__FUNCTION__);
	
	if (!GTK_WIDGET_VISIBLE (lookup_widget (windowTop, "hbox4")))
	{
		gtk_widget_show (lookup_widget (windowTop,
						"label_tastatur1_dummy"));
		gtk_widget_show (lookup_widget (windowTop,
						"label_tastatur_dummy"));
	}
	
	gtk_set_mouse_position (wid, 550, 50);
	if (keyboard1)
	{
		activate_pengudrive (__FUNCTION__ ,39, tastatur.pos);
		gtk_widget_show (lookup_widget (windowTop,
						"eventbox_tastatur1"));
	}
	else 
	{
		activate_pengudrive (__FUNCTION__ ,43, tastatur.pos);
		gtk_widget_show (lookup_widget (windowTop,
						"eventbox_tastatur"));
	}
	gtk_widget_grab_default (tastatur.active);
	
	keyboard_open = TRUE;
}

void hide_tastatur (GtkWidget *wid)
{
	disable_pengudrive (__FUNCTION__);
	gtk_widget_hide (lookup_widget (windowTop,
					"label_tastatur_dummy"));
	gtk_widget_hide (lookup_widget (windowTop,
					"label_tastatur1_dummy"));
	if (keyboard1)
	    gtk_widget_hide (lookup_widget (windowTop,
					    "eventbox_tastatur1"));
	else
	    gtk_widget_hide (lookup_widget (windowTop,
					    "eventbox_tastatur"));
	gtk_widget_hide (combo_list);
	keyboard_open = FALSE;
}

void
on_eventbox_tastatur_show              (GtkWidget       *widget,
                                        gpointer         user_data)
{
}

gboolean
on_eventbox_tastatur_enter_notify_event        (GtkWidget       *widget,
						    GdkEventCrossing *event,
						    gpointer         user_data)
{
	g_print ("Tastatur entered\n");
	return FALSE;
}


gboolean
on_eventbox_tastatur_leave_notify_event        (GtkWidget       *widget,
						    GdkEventCrossing *event,
						    gpointer         user_data)
{
	g_print ("Tastatur left\n");
	return FALSE;
}


gboolean
on_eventbox_tastatur_scroll_event              (GtkWidget       *widget,
						    GdkEvent        *event,
						    gpointer         user_data)
{
	guint button = event->scroll.direction;
	
	g_print ("%s: Button %d activate: ", __FUNCTION__, button);
	switch (button)
	{
		
	case GDK_SCROLL_UP: 
		if (tastatur.prev) 
		{
			gtk_set_mouse_position (windowTop, 
						tastatur.px, tastatur.py);
			g_print ("prev\n");
		}
		else g_print ("ignored (first!)\n");
		break;
		
	case GDK_SCROLL_DOWN: 
		if (tastatur.next) 
		{
			gtk_set_mouse_position (windowTop, 
						tastatur.nx, tastatur.ny);
			g_print ("next\n");
		}
		else g_print ("ignored (lastst!)\n");
		break;
		
	default: g_print ("ignored\n");
	}
	
	return FALSE;
}

gboolean
on_eventbox_tastatur_button_press_event	(GtkWidget       *widget,
					 GdkEventButton  *event,
					 gpointer         user_data)
{
	g_signal_emit_by_name ((gpointer) tastatur.active, "activate");
	return FALSE;
}

/* }}} */

/* {{{ Keyboard 0: capital letters */

void
on_tast_A_enter (GtkButton *button, gpointer user_data)
{
	tastatur.active = GTK_WIDGET (button);
	tastatur.prev = NULL;
	tastatur.next = lookup_widget (GTK_WIDGET (button), "tast_B");
	tastatur.px = 550;
	tastatur.py = 50;
	tastatur.nx = 570;
	tastatur.ny = 50;
gtk_widget_grab_default (tastatur.active);
	tastatur.pos = 0;
	activate_pengudrive (__FUNCTION__ ,43, tastatur.pos);
}

void
on_tast_A_clicked (GtkButton *button, gpointer user_data)
{
	add_char (tastatur.ref, "A", 1);
}

void
on_tast_B_enter (GtkButton *button, gpointer user_data)
{
	tastatur.active = GTK_WIDGET (button);
	tastatur.prev = lookup_widget (GTK_WIDGET (button), "tast_A");
	tastatur.next = lookup_widget (GTK_WIDGET (button), "tast_C");
	tastatur.px = 550;
	tastatur.py = 50;
	tastatur.nx = 590;
	tastatur.ny = 50;
gtk_widget_grab_default (tastatur.active);
	tastatur.pos = 1;
	activate_pengudrive (__FUNCTION__ ,43, tastatur.pos);
}

void
on_tast_B_clicked (GtkButton *button, gpointer user_data)
{
	add_char (tastatur.ref, "B", 1);
}

void
on_tast_C_enter (GtkButton *button, gpointer user_data)
{
	tastatur.active = GTK_WIDGET (button);
	tastatur.prev = lookup_widget (GTK_WIDGET (button), "tast_B");
	tastatur.next = lookup_widget (GTK_WIDGET (button), "tast_D");
	tastatur.px = 570;
	tastatur.py = 50;
	tastatur.nx = 610;
	tastatur.ny = 50;
gtk_widget_grab_default (tastatur.active);
	tastatur.pos = 2;
	activate_pengudrive (__FUNCTION__ ,43, tastatur.pos);
}

void
on_tast_C_clicked (GtkButton *button, gpointer user_data)
{
	add_char (tastatur.ref, "C", 1);
}

void
on_tast_D_enter (GtkButton *button, gpointer user_data)
{
	tastatur.active = GTK_WIDGET (button);
	tastatur.prev = lookup_widget (GTK_WIDGET (button), "tast_C");
	tastatur.next = lookup_widget (GTK_WIDGET (button), "tast_E");
	tastatur.px = 590;
	tastatur.py = 50;
	tastatur.nx = 630;
	tastatur.ny = 50;
gtk_widget_grab_default (tastatur.active);
	tastatur.pos = 3;
	activate_pengudrive (__FUNCTION__ ,43, tastatur.pos);
}

void
on_tast_D_clicked (GtkButton *button, gpointer user_data)
{
	add_char (tastatur.ref, "D", 1);
}


void
on_tast_E_enter (GtkButton *button, gpointer user_data)
{
	tastatur.active = GTK_WIDGET (button);
	tastatur.prev = lookup_widget (GTK_WIDGET (button), "tast_D");
	tastatur.next = lookup_widget (GTK_WIDGET (button), "tast_F");
	tastatur.px = 610;
	tastatur.py = 50;
	tastatur.nx = 550;
	tastatur.ny = 70;
gtk_widget_grab_default (tastatur.active);
	tastatur.pos = 4;
	activate_pengudrive (__FUNCTION__ ,43, tastatur.pos);
}

void
on_tast_E_clicked (GtkButton *button, gpointer user_data)
{
	add_char (tastatur.ref, "E", 1);
}

void
on_tast_F_enter (GtkButton *button, gpointer user_data)
{
	tastatur.active = GTK_WIDGET (button);
	tastatur.prev = lookup_widget (GTK_WIDGET (button), "tast_E");
	tastatur.next = lookup_widget (GTK_WIDGET (button), "tast_G");
	tastatur.px = 630;
	tastatur.py = 50;
	tastatur.nx = 570;
	tastatur.ny = 70;
gtk_widget_grab_default (tastatur.active);
	tastatur.pos = 5;
	activate_pengudrive (__FUNCTION__ ,43, tastatur.pos);
}

void
on_tast_F_clicked (GtkButton *button, gpointer user_data)
{
	add_char (tastatur.ref, "F", 1);
}

void
on_tast_G_enter                     (GtkButton       *button,
                                        gpointer         user_data)
{
	tastatur.active = GTK_WIDGET (button);
	tastatur.prev = lookup_widget (GTK_WIDGET (button), "tast_F");
	tastatur.next = lookup_widget (GTK_WIDGET (button), "tast_H");
	tastatur.px = 550;
	tastatur.py = 70;
	tastatur.nx = 590;
	tastatur.ny = 70;
gtk_widget_grab_default (tastatur.active);
	tastatur.pos = 6;
	activate_pengudrive (__FUNCTION__ ,43, tastatur.pos);
}

void
on_tast_G_clicked                      (GtkButton       *button,
                                        gpointer         user_data)
{
	add_char (tastatur.ref, "G", 1);
}

void
on_tast_H_enter                     (GtkButton       *button,
                                        gpointer         user_data)
{
	tastatur.active = GTK_WIDGET (button);
	tastatur.prev = lookup_widget (GTK_WIDGET (button), "tast_G");
	tastatur.next = lookup_widget (GTK_WIDGET (button), "tast_I");
	tastatur.px = 570;
	tastatur.py = 70;
	tastatur.nx = 610;
	tastatur.ny = 70;
gtk_widget_grab_default (tastatur.active);
	tastatur.pos = 7;
	activate_pengudrive (__FUNCTION__ ,43, tastatur.pos);
}

void
on_tast_H_clicked                      (GtkButton       *button,
                                        gpointer         user_data)
{
	add_char (tastatur.ref, "H", 1);
}

void
on_tast_I_enter                     (GtkButton       *button,
                                        gpointer         user_data)
{
	tastatur.active = GTK_WIDGET (button);
	tastatur.prev = lookup_widget (GTK_WIDGET (button), "tast_H");
	tastatur.next = lookup_widget (GTK_WIDGET (button), "tast_J");
	tastatur.px = 590;
	tastatur.py = 70;
	tastatur.nx = 630;
	tastatur.ny = 70;
gtk_widget_grab_default (tastatur.active);
	tastatur.pos = 8;
	activate_pengudrive (__FUNCTION__ ,43, tastatur.pos);
}

void
on_tast_I_clicked                      (GtkButton       *button,
                                        gpointer         user_data)
{
	add_char (tastatur.ref, "I", 1);
}


void
on_tast_J_enter                     (GtkButton       *button,
                                        gpointer         user_data)
{
	tastatur.active = GTK_WIDGET (button);
	tastatur.prev = lookup_widget (GTK_WIDGET (button), "tast_I");
	tastatur.next = lookup_widget (GTK_WIDGET (button), "tast_K");
	tastatur.px = 610;
	tastatur.py = 70;
	tastatur.nx = 550;
	tastatur.ny = 90;
gtk_widget_grab_default (tastatur.active);
	tastatur.pos = 9;
	activate_pengudrive (__FUNCTION__ ,43, tastatur.pos);
}

void
on_tast_J_clicked                      (GtkButton       *button,
                                        gpointer         user_data)
{
	add_char (tastatur.ref, "J", 1);
}

void
on_tast_K_enter                     (GtkButton       *button,
                                        gpointer         user_data)
{
	tastatur.active = GTK_WIDGET (button);
	tastatur.prev = lookup_widget (GTK_WIDGET (button), "tast_J");
	tastatur.next = lookup_widget (GTK_WIDGET (button), "tast_L");
	tastatur.px = 630;
	tastatur.py = 70;
	tastatur.nx = 570;
	tastatur.ny = 90;
gtk_widget_grab_default (tastatur.active);
	tastatur.pos = 10;
	activate_pengudrive (__FUNCTION__ ,43, tastatur.pos);
}

void
on_tast_K_clicked                      (GtkButton       *button,
                                        gpointer         user_data)
{
	add_char (tastatur.ref, "K", 1);
}

void
on_tast_L_enter                     (GtkButton       *button,
                                        gpointer         user_data)
{
	tastatur.active = GTK_WIDGET (button);
	tastatur.prev = lookup_widget (GTK_WIDGET (button), "tast_K");
	tastatur.next = lookup_widget (GTK_WIDGET (button), "tast_M");
	tastatur.px = 550;
	tastatur.py = 90;
	tastatur.nx = 590;
	tastatur.ny = 90;
gtk_widget_grab_default (tastatur.active);
	tastatur.pos = 11;
	activate_pengudrive (__FUNCTION__ ,43, tastatur.pos);
}

void
on_tast_L_clicked                      (GtkButton       *button,
                                        gpointer         user_data)
{
	add_char (tastatur.ref, "L", 1);
}

void
on_tast_M_enter                     (GtkButton       *button,
                                        gpointer         user_data)
{
	tastatur.active = GTK_WIDGET (button);
	tastatur.prev = lookup_widget (GTK_WIDGET (button), "tast_L");
	tastatur.next = lookup_widget (GTK_WIDGET (button), "tast_N");
	tastatur.px = 570;
	tastatur.py = 90;
	tastatur.nx = 610;
	tastatur.ny = 90;
gtk_widget_grab_default (tastatur.active);
	tastatur.pos = 12;
	activate_pengudrive (__FUNCTION__ ,43, tastatur.pos);
}

void
on_tast_M_clicked                      (GtkButton       *button,
                                        gpointer         user_data)
{
	add_char (tastatur.ref, "M", 1);
}

void
on_tast_N_enter                     (GtkButton       *button,
                                        gpointer         user_data)
{
	tastatur.active = GTK_WIDGET (button);
	tastatur.prev = lookup_widget (GTK_WIDGET (button), "tast_M");
	tastatur.next = lookup_widget (GTK_WIDGET (button), "tast_O");
	tastatur.px = 590;
	tastatur.py = 90;
	tastatur.nx = 630;
	tastatur.ny = 90;
gtk_widget_grab_default (tastatur.active);
	tastatur.pos = 13;
	activate_pengudrive (__FUNCTION__ ,43, tastatur.pos);
}

void
on_tast_N_clicked                      (GtkButton       *button,
                                        gpointer         user_data)
{
	add_char (tastatur.ref, "N", 1);
}

void
on_tast_O_enter                     (GtkButton       *button,
                                        gpointer         user_data)
{
	tastatur.active = GTK_WIDGET (button);
	tastatur.prev = lookup_widget (GTK_WIDGET (button), "tast_N");
	tastatur.next = lookup_widget (GTK_WIDGET (button), "tast_P");
	tastatur.px = 610;
	tastatur.py = 90;
	tastatur.nx = 550;
	tastatur.ny = 110;
gtk_widget_grab_default (tastatur.active);
	tastatur.pos = 14;
	activate_pengudrive (__FUNCTION__ ,43, tastatur.pos);
}

void
on_tast_O_clicked                      (GtkButton       *button,
                                        gpointer         user_data)
{
	add_char (tastatur.ref, "O", 1);
}

void
on_tast_P_enter                     (GtkButton       *button,
                                        gpointer         user_data)
{
	tastatur.active = GTK_WIDGET (button);
	tastatur.prev = lookup_widget (GTK_WIDGET (button), "tast_O");
	tastatur.next = lookup_widget (GTK_WIDGET (button), "tast_Q");
	tastatur.px = 630;
	tastatur.py = 90;
	tastatur.nx = 570;
	tastatur.ny = 110;
gtk_widget_grab_default (tastatur.active);
	tastatur.pos = 15;
	activate_pengudrive (__FUNCTION__ ,43, tastatur.pos);
}

void
on_tast_P_clicked                      (GtkButton       *button,
                                        gpointer         user_data)
{
	add_char (tastatur.ref, "P", 1);
}

void
on_tast_Q_enter                     (GtkButton       *button,
                                        gpointer         user_data)
{
	tastatur.active = GTK_WIDGET (button);
	tastatur.prev = lookup_widget (GTK_WIDGET (button), "tast_P");
	tastatur.next = lookup_widget (GTK_WIDGET (button), "tast_R");
	tastatur.px = 550;
	tastatur.py = 110;
	tastatur.nx = 590;
	tastatur.ny = 110;
gtk_widget_grab_default (tastatur.active);
	tastatur.pos = 16;
	activate_pengudrive (__FUNCTION__ ,43, tastatur.pos);
}

void
on_tast_Q_clicked                      (GtkButton       *button,
                                        gpointer         user_data)
{
	add_char (tastatur.ref, "Q", 1);
}

void
on_tast_R_enter                     (GtkButton       *button,
                                        gpointer         user_data)
{
	tastatur.active = GTK_WIDGET (button);
	tastatur.prev = lookup_widget (GTK_WIDGET (button), "tast_Q");
	tastatur.next = lookup_widget (GTK_WIDGET (button), "tast_S");
	tastatur.px = 570;
	tastatur.py = 110;
	tastatur.nx = 610;
	tastatur.ny = 110;
gtk_widget_grab_default (tastatur.active);
	tastatur.pos = 17;
	activate_pengudrive (__FUNCTION__ ,43, tastatur.pos);
}

void
on_tast_R_clicked                      (GtkButton       *button,
                                        gpointer         user_data)
{
	add_char (tastatur.ref, "R", 1);
}

void
on_tast_S_enter                     (GtkButton       *button,
                                        gpointer         user_data)
{
	tastatur.active = GTK_WIDGET (button);
	tastatur.prev = lookup_widget (GTK_WIDGET (button), "tast_R");
	tastatur.next = lookup_widget (GTK_WIDGET (button), "tast_T");
	tastatur.px = 590;
	tastatur.py = 110;
	tastatur.nx = 630;
	tastatur.ny = 110;
gtk_widget_grab_default (tastatur.active);
	tastatur.pos = 18;
	activate_pengudrive (__FUNCTION__ ,43, tastatur.pos);
}

void
on_tast_S_clicked                      (GtkButton       *button,
                                        gpointer         user_data)
{
	add_char (tastatur.ref, "S", 1);
}

void
on_tast_T_enter                     (GtkButton       *button,
                                        gpointer         user_data)
{
	tastatur.active = GTK_WIDGET (button);
	tastatur.prev = lookup_widget (GTK_WIDGET (button), "tast_S");
	tastatur.next = lookup_widget (GTK_WIDGET (button), "tast_U");
	tastatur.px = 610;
	tastatur.py = 110;
	tastatur.nx = 550;
	tastatur.ny = 130;
gtk_widget_grab_default (tastatur.active);
	tastatur.pos = 19;
	activate_pengudrive (__FUNCTION__ ,43, tastatur.pos);
}

void
on_tast_T_clicked                      (GtkButton       *button,
                                        gpointer         user_data)
{
	add_char (tastatur.ref, "T", 1);
}

void
on_tast_U_enter                     (GtkButton       *button,
                                        gpointer         user_data)
{
	tastatur.active = GTK_WIDGET (button);
	tastatur.prev = lookup_widget (GTK_WIDGET (button), "tast_T");
	tastatur.next = lookup_widget (GTK_WIDGET (button), "tast_V");
	tastatur.px = 630;
	tastatur.py = 110;
	tastatur.nx = 570;
	tastatur.ny = 130;
gtk_widget_grab_default (tastatur.active);
	tastatur.pos = 20;
	activate_pengudrive (__FUNCTION__ ,43, tastatur.pos);
}

void
on_tast_U_clicked                      (GtkButton       *button,
                                        gpointer         user_data)
{
	add_char (tastatur.ref, "U", 1);
}

void
on_tast_V_enter                     (GtkButton       *button,
                                        gpointer         user_data)
{
	tastatur.active = GTK_WIDGET (button);
	tastatur.prev = lookup_widget (GTK_WIDGET (button), "tast_U");
	tastatur.next = lookup_widget (GTK_WIDGET (button), "tast_W");
	tastatur.px = 550;
	tastatur.py = 130;
	tastatur.nx = 590;
	tastatur.ny = 130;
gtk_widget_grab_default (tastatur.active);
	tastatur.pos = 21;
	activate_pengudrive (__FUNCTION__ ,43, tastatur.pos);
}

void
on_tast_V_clicked                      (GtkButton       *button,
                                        gpointer         user_data)
{
	add_char (tastatur.ref, "V", 1);
}

void
on_tast_W_enter                     (GtkButton       *button,
                                        gpointer         user_data)
{
	tastatur.active = GTK_WIDGET (button);
	tastatur.prev = lookup_widget (GTK_WIDGET (button), "tast_V");
	tastatur.next = lookup_widget (GTK_WIDGET (button), "tast_X");
	tastatur.px = 570;
	tastatur.py = 130;
	tastatur.nx = 610;
	tastatur.ny = 130;
gtk_widget_grab_default (tastatur.active);
	tastatur.pos = 22;
	activate_pengudrive (__FUNCTION__ ,43, tastatur.pos);
}

void
on_tast_W_clicked                      (GtkButton       *button,
                                        gpointer         user_data)
{
	add_char (tastatur.ref, "W", 1);
}

void
on_tast_X_enter                     (GtkButton       *button,
                                        gpointer         user_data)
{
	tastatur.active = GTK_WIDGET (button);
	tastatur.prev = lookup_widget (GTK_WIDGET (button), "tast_W");
	tastatur.next = lookup_widget (GTK_WIDGET (button), "tast_Y");
	tastatur.px = 590;
	tastatur.py = 130;
	tastatur.nx = 630;
	tastatur.ny = 130;
gtk_widget_grab_default (tastatur.active);
	tastatur.pos = 23;
	activate_pengudrive (__FUNCTION__ ,43, tastatur.pos);
}

void
on_tast_X_clicked                      (GtkButton       *button,
                                        gpointer         user_data)
{
	add_char (tastatur.ref, "X", 1);
}

void
on_tast_Y_enter                     (GtkButton       *button,
                                        gpointer         user_data)
{
	tastatur.active = GTK_WIDGET (button);
	tastatur.prev = lookup_widget (GTK_WIDGET (button), "tast_X");
	tastatur.next = lookup_widget (GTK_WIDGET (button), "tast_Z");
	tastatur.px = 610;
	tastatur.py = 130;
	tastatur.nx = 550;
	tastatur.ny = 150;
gtk_widget_grab_default (tastatur.active);
	tastatur.pos = 24;
	activate_pengudrive (__FUNCTION__ ,43, tastatur.pos);
}

void
on_tast_Y_clicked                      (GtkButton       *button,
                                        gpointer         user_data)
{
	add_char (tastatur.ref, "Y", 1);
}

void
on_tast_Z_enter                     (GtkButton       *button,
                                        gpointer         user_data)
{
	tastatur.active = GTK_WIDGET (button);
	tastatur.prev = lookup_widget (GTK_WIDGET (button), "tast_Y");
	tastatur.next = lookup_widget (GTK_WIDGET (button), "tast_leer");
	tastatur.px = 630;
	tastatur.py = 130;
	tastatur.nx = 570;
	tastatur.ny = 150;
gtk_widget_grab_default (tastatur.active);
	tastatur.pos = 25;
	activate_pengudrive (__FUNCTION__ ,43, tastatur.pos);
}

void
on_tast_Z_clicked                      (GtkButton       *button,
                                        gpointer         user_data)
{
	add_char (tastatur.ref, "Z", 1);
}

void
on_tast_leer_enter                  (GtkButton       *button,
                                        gpointer         user_data)
{
	tastatur.active = GTK_WIDGET (button);
	tastatur.prev = lookup_widget (GTK_WIDGET (button), "tast_Z");
	tastatur.next = lookup_widget (GTK_WIDGET (button), "tast_slash");
	tastatur.px = 550;
	tastatur.py = 150;
	tastatur.nx = 590;
	tastatur.ny = 150;
gtk_widget_grab_default (tastatur.active);
	tastatur.pos = 26;
	activate_pengudrive (__FUNCTION__ ,43, tastatur.pos);
}

void
on_tast_leer_clicked                   (GtkButton       *button,
                                        gpointer         user_data)
{
	add_char (tastatur.ref, " ", 1);
}

void
on_tast_slash_enter               (GtkButton       *button,
                                        gpointer         user_data)
{
	tastatur.active = GTK_WIDGET (button);
	tastatur.prev = lookup_widget (GTK_WIDGET (button), "tast_leer");
	tastatur.next = lookup_widget (GTK_WIDGET (button), "tast_dot");
	tastatur.px = 570;
	tastatur.py = 150;
	tastatur.nx = 610;
	tastatur.ny = 150;
gtk_widget_grab_default (tastatur.active);
	tastatur.pos = 27;
	activate_pengudrive (__FUNCTION__ ,43, tastatur.pos);
}

void
on_tast_slash_clicked                (GtkButton       *button,
                                        gpointer         user_data)
{
	add_char (tastatur.ref, "/", 1);
}

void
on_tast_dot_enter                 (GtkButton       *button,
                                        gpointer         user_data)
{
	tastatur.active = GTK_WIDGET (button);
	tastatur.prev = lookup_widget (GTK_WIDGET (button), "tast_slash");
	tastatur.next = lookup_widget (GTK_WIDGET (button), "tast_vor");
	tastatur.px = 590;
	tastatur.py = 150;
	tastatur.nx = 630;
	tastatur.ny = 150;
gtk_widget_grab_default (tastatur.active);
	tastatur.pos = 28;
	activate_pengudrive (__FUNCTION__ ,43, tastatur.pos);
}

void
on_tast_dot_clicked                  (GtkButton       *button,
                                        gpointer         user_data)
{
	add_char (tastatur.ref, ".", 1);
}

void
on_tast_vor_enter                   (GtkButton       *button,
                                        gpointer         user_data)
{
	GtkWidget *image_setup;
	tastatur.active = GTK_WIDGET (button);
	tastatur.prev = lookup_widget (GTK_WIDGET (button), "tast_dot");
	tastatur.next = lookup_widget (GTK_WIDGET (button), "tast_0");
	tastatur.px = 610;
	tastatur.py = 150;
	tastatur.nx = 550;
	tastatur.ny = 170;
gtk_widget_grab_default (tastatur.active);
	tastatur.pos = 29;
	activate_pengudrive (__FUNCTION__ ,43, tastatur.pos);
}

void
on_tast_vor_clicked                    (GtkButton       *button,
                                        gpointer         user_data)
{
	g_print ("Change to 1\n");
	
	activate_pengudrive (__FUNCTION__ ,39, tastatur.pos);
	gtk_widget_hide (lookup_widget (GTK_WIDGET (button), 
					"eventbox_tastatur"));
	gtk_widget_show (lookup_widget (GTK_WIDGET (button), 
					"eventbox_tastatur1"));
	g_signal_emit_by_name ((gpointer) lookup_widget (GTK_WIDGET (button), 
							 "tast_vor1"),
			       "notify");
	keyboard1 = TRUE;
	
}

void
on_tast_www_enter                 (GtkButton       *button,
                                        gpointer         user_data)
{
	tastatur.active = GTK_WIDGET (button);
	tastatur.prev = lookup_widget (GTK_WIDGET (button), "tast_vor1");
	tastatur.next = lookup_widget (GTK_WIDGET (button), "tast_de");
	tastatur.px = 630;
	tastatur.py = 150;
	tastatur.nx = 600;
	tastatur.ny = 170;
gtk_widget_grab_default (tastatur.active);
	tastatur.pos = 30;
	activate_pengudrive (__FUNCTION__ ,43, tastatur.pos);
}


void
on_tast_www_clicked                  (GtkButton       *button,
                                        gpointer         user_data)
{
	add_char (tastatur.ref, "www.", 4);
}

void
on_tast_de_enter                  (GtkButton       *button,
                                        gpointer         user_data)
{
	tastatur.active = GTK_WIDGET (button);
	tastatur.prev = lookup_widget (GTK_WIDGET (button), "tast_www");
	tastatur.next = lookup_widget (GTK_WIDGET (button), "tast_colon");
	tastatur.px = 560;
	tastatur.py = 170;
	tastatur.nx = 630;
	tastatur.ny = 170;
gtk_widget_grab_default (tastatur.active);
	tastatur.pos = 31;
	activate_pengudrive (__FUNCTION__ ,43, tastatur.pos);
}


void
on_tast_de_clicked                   (GtkButton       *button,
                                        gpointer         user_data)
{
	add_char (tastatur.ref, ".de", 3);
}

void
on_tast_colon_enter               (GtkButton       *button,
                                        gpointer         user_data)
{
	tastatur.active = GTK_WIDGET (button);
	tastatur.prev = lookup_widget (GTK_WIDGET (button), "tast_de");
	tastatur.next = lookup_widget (GTK_WIDGET (button), "tast_com");
	tastatur.px = 600;
	tastatur.py = 170;
	tastatur.nx = 560;
	tastatur.ny = 190;
gtk_widget_grab_default (tastatur.active);
	tastatur.pos = 32;
	activate_pengudrive (__FUNCTION__ ,43, tastatur.pos);
}

void
on_tast_colon_clicked                (GtkButton       *button,
                                        gpointer         user_data)
{
	add_char (tastatur.ref, ":", 1);
}

void
on_tast_com_enter                 (GtkButton       *button,
                                        gpointer         user_data)
{
	tastatur.active = GTK_WIDGET (button);
	tastatur.prev = lookup_widget (GTK_WIDGET (button), "tast_colon");
	tastatur.next = lookup_widget (GTK_WIDGET (button), "tast_org");
	tastatur.px = 630;
	tastatur.py = 170;
	tastatur.nx = 600;
	tastatur.ny = 190;
gtk_widget_grab_default (tastatur.active);
	tastatur.pos = 33;
	activate_pengudrive (__FUNCTION__ ,43, tastatur.pos);
}

void
on_tast_com_clicked                  (GtkButton       *button,
                                        gpointer         user_data)
{
	add_char (tastatur.ref, ".com", 4);
}

void
on_tast_org_enter                 (GtkButton       *button,
                                        gpointer         user_data)
{
	tastatur.active = GTK_WIDGET (button);
	tastatur.prev = lookup_widget (GTK_WIDGET (button), "tast_com");
	tastatur.next = lookup_widget (GTK_WIDGET (button), "tast_dash");
	tastatur.px = 560;
	tastatur.py = 190;
	tastatur.nx = 630;
	tastatur.ny = 190;
gtk_widget_grab_default (tastatur.active);
	tastatur.pos = 34;
	activate_pengudrive (__FUNCTION__ ,43, tastatur.pos);
}

void
on_tast_org_clicked                  (GtkButton       *button,
                                        gpointer         user_data)
{
	add_char (tastatur.ref, ".org", 4);
}

void
on_tast_dash_enter                (GtkButton       *button,
                                        gpointer         user_data)
{
	tastatur.active = GTK_WIDGET (button);
	tastatur.prev = lookup_widget (GTK_WIDGET (button), "tast_org");
	tastatur.next = lookup_widget (GTK_WIDGET (button), "tast_enter1");
	tastatur.px = 600;
	tastatur.py = 190;
	tastatur.nx = 541;
	tastatur.ny = 230;
gtk_widget_grab_default (tastatur.active);
	tastatur.pos = 35;
	activate_pengudrive (__FUNCTION__ ,43, tastatur.pos);
}


void
on_tast_dash_clicked                 (GtkButton       *button,
                                        gpointer         user_data)
{
	add_char (tastatur.ref, "-", 1);
}

void
on_tast_enter_enter                 (GtkButton       *button,
                                        gpointer         user_data)
{
	tastatur.active = GTK_WIDGET (button);
	tastatur.prev = lookup_widget (GTK_WIDGET (button), "tast_9");
	tastatur.next = lookup_widget (GTK_WIDGET (button), "tast_esc");
	tastatur.px = 630;
	tastatur.py = 190;
	tastatur.nx = 574;
	tastatur.ny = 230;
gtk_widget_grab_default (tastatur.active);
	tastatur.pos = 40;
	activate_pengudrive (__FUNCTION__ ,43, tastatur.pos);
}

void
on_tast_enter_clicked                  (GtkButton       *button,
                                        gpointer         user_data)
{
	hide_tastatur (lookup_widget (GTK_WIDGET (button),
					"eventbox_tastatur"));
	disable_pengudrive (__FUNCTION__);
	tastatur.fix_mouse = FALSE;
	atk_action_do_action (tastatur.ref, 0);
}

void
on_tast_esc_enter                 (GtkButton       *button,
                                        gpointer         user_data)
{
	tastatur.active = GTK_WIDGET (button);
	tastatur.prev = lookup_widget (GTK_WIDGET (button), "tast_enter");
	tastatur.next = lookup_widget (GTK_WIDGET (button), "tast_back");
	tastatur.px = 541;
	tastatur.py = 230;
	tastatur.nx = 607;
	tastatur.ny = 230;
gtk_widget_grab_default (tastatur.active);
	tastatur.pos = 41;
	activate_pengudrive (__FUNCTION__ ,43, tastatur.pos);
}

void
on_tast_esc_clicked                  (GtkButton       *button,
                                        gpointer         user_data)
{
	gint l;
	gchar *text;
	gtk_widget_hide (lookup_widget (GTK_WIDGET (button), 
					"eventbox_tastatur"));
	if (ATK_IS_EDITABLE_TEXT (tastatur.ref))
	{
		text = atk_text_get_text (ATK_TEXT (tastatur.ref), 0, -1);
		if (text)
		{
			l = strlen (text);
			atk_editable_text_delete_text (ATK_EDITABLE_TEXT (tastatur.ref), 
						       0, l);
		}
	}
	tastatur.fix_mouse = FALSE;
	disable_pengudrive (__FUNCTION__);
}

/* }}} */

/* {{{ Gecko Engine */

gboolean
on_eventbox_gecko_enter_notify_event   (GtkWidget       *widget,
                                        GdkEventCrossing *event,
                                        gpointer         user_data)
{
	static gboolean first_use = TRUE;
	if (!toolbar_load && !toolbar_all && !first_use && 
	    !keyboard_open && !GTK_WIDGET_VISIBLE(combo_list)) 
	    gtk_widget_hide (lookup_widget (widget, "hbox4"));
	
	g_print ("gecko entered\n");
	first_use = FALSE;
	return FALSE;
}


gboolean
on_eventbox_gecko_leave_notify_event   (GtkWidget       *widget,
                                        GdkEventCrossing *event,
                                        gpointer         user_data)
{
	g_print ("gecko left\n");
	return FALSE;
}

gboolean
on_gecko_focus                         (GtkWidget       *widget,
                                        GtkDirectionType  direction,
                                        gpointer         user_data)
{
	return FALSE;
}

/* }}} */

/* {{{ Menu Buttons */

void
on_setup_enter                      (GtkButton       *button,
                                        gpointer         user_data)
{
	toolbar.active = GTK_WIDGET (button);
	toolbar.prev = NULL;
	toolbar.next = lookup_widget (GTK_WIDGET (button), "online");
	gtk_set_mouse_position (GTK_WIDGET (button), 30, 25);
	activate_pengudrive (__FUNCTION__ ,6, 0);
//	chg_pixmap (button, "image_setup", "setup_cursor.xpm");
}


void
on_setup_leave                         (GtkButton       *button,
					gpointer         user_data)
{
//	chg_pixmap (button, "image_setup", "setup.xpm");
}


void
on_setup_clicked                       (GtkButton       *button,
                                        gpointer         user_data)
{
//	chg_pixmap (GTK_WIDGET (button), "image_setup", "setup_gedrueckt.xpm");
	gtk_widget_show (lookup_widget (GTK_WIDGET (button), "eventbox_setup"));
	gtk_widget_hide (lookup_widget (GTK_WIDGET (button), "eventbox_gecko"));
//	chg_pixmap (GTK_WIDGET (button), "image_setup", "setup_gedrueckt.xpm");
}


void
on_setup_released                      (GtkButton       *button,
                                        gpointer         user_data)
{
//	chg_pixmap (GTK_WIDGET (button), "image_setup", "setup_cursor.xpm");
}


void
on_online_enter                     (GtkButton       *button,
                                        gpointer         user_data)
{
	toolbar.active = GTK_WIDGET (button);
	toolbar.prev = lookup_widget (GTK_WIDGET (button), "setup");
	toolbar.next = lookup_widget (GTK_WIDGET (button), "back");
	gtk_set_mouse_position (GTK_WIDGET (button), 100, 25);
	activate_pengudrive (__FUNCTION__ ,6, 1);
//	if (link_status)
//		chg_pixmap (button, "image_online", "online_cursor.xpm");
//	else
//		chg_pixmap (button, "image_online", "gprs_cursor.xpm");
}


void
on_online_leave                       (GtkButton       *button,
				       gpointer         user_data)
{
//	if (link_status)
//		chg_pixmap (button, "image_online", "online.xpm");
//	else
//		chg_pixmap (button, "image_online", "gprs.xpm");
}


void
on_online_clicked                      (GtkButton       *button,
                                        gpointer         user_data)
{
	if (link_status)
	{
		if (!nogprs) 
		{
			system ("/etc/init.d/ppp-link stop");
//			system ("/etc/init.d/ppp-daemon stop");
		}
		link_status = FALSE;
		chg_pixmap (GTK_WIDGET (button), "image_online", "gprs.xpm");
		gtk_widget_show (lookup_widget (windowTop, "ampel_rot"));
		gtk_widget_hide (lookup_widget (windowTop, "ampel_gruen"));
	}
	else
	{
		if (!nogprs)
		{
			system ("/etc/init.d/ppp-link start");
//			system ("/etc/init.d/ppp-daemon start");
		}
		link_status = TRUE;
		chg_pixmap (GTK_WIDGET (button), "image_online", "offline.xpm");
		gtk_widget_hide (lookup_widget (windowTop, "ampel_rot"));
		gtk_widget_show (lookup_widget (windowTop, "ampel_gruen"));
	}
}


void
on_online_released                     (GtkButton       *button,
                                        gpointer         user_data)
{
//	if (link_status)
//	    chg_pixmap (GTK_WIDGET (button), "image_online", 
//			"online_cursor.xpm");
//	else
//	    chg_pixmap (GTK_WIDGET (button), "image_online", 
//			"gprs_cursor.xpm");
}


void
on_back_enter                       (GtkButton       *button,
                                        gpointer         user_data)
{
	toolbar.active = GTK_WIDGET (button);
	toolbar.prev = lookup_widget (GTK_WIDGET (button), "online");
	toolbar.next = lookup_widget (GTK_WIDGET (button), "forward");
	gtk_set_mouse_position (GTK_WIDGET (button), 150, 25);
	activate_pengudrive (__FUNCTION__ ,6, 2);
//	chg_pixmap (button, "image_back", "zurueck_cursor.xpm");
}


void
on_back_leave                          (GtkButton       *button,
					gpointer         user_data)
{
	chg_pixmap (button, "image_back", "zurueck.xpm");
}


void
on_back_clicked                        (GtkButton       *button,
                                        gpointer         user_data)
{
	gtk_moz_embed_go_back (GTK_MOZ_EMBED (lookup_widget (GTK_WIDGET (button), 
							     "gecko")));
//	chg_pixmap (GTK_WIDGET (button), "image_back", 
//		    "zurueck_gedrueckt.xpm");
}


void
on_back_released                       (GtkButton       *button,
                                        gpointer         user_data)
{
//	chg_pixmap (GTK_WIDGET (button), "image_back", 
//		    "zurueck_cursor.xpm");
}


void
on_forward_enter                    (GtkButton       *button,
                                        gpointer         user_data)
{
	toolbar.active = GTK_WIDGET (button);
	toolbar.prev = lookup_widget (GTK_WIDGET (button), "back");
	toolbar.next = lookup_widget (GTK_WIDGET (button), "reload");
	gtk_set_mouse_position (GTK_WIDGET (button), 200, 25);
	activate_pengudrive (__FUNCTION__ ,6, 3);
//	chg_pixmap (button, "image_forward", "vor_cursor.xpm");
}


void
on_forward_leave                       (GtkButton       *button,
					gpointer         user_data)
{
//	chg_pixmap (button, "image_forward", "vor.xpm");
}


void
on_forward_clicked                     (GtkButton       *button,
                                        gpointer         user_data)
{
	gtk_moz_embed_go_forward (GTK_MOZ_EMBED (lookup_widget (GTK_WIDGET (button), 
								"gecko")));

//	chg_pixmap (GTK_WIDGET (button), "image_forward", 
//		                        "vor_gedrueckt.xpm");
}


void
on_forward_released                    (GtkButton       *button,
                                        gpointer         user_data)
{
//	chg_pixmap (GTK_WIDGET (button), "image_forward",
//		    "vor_gedrueckt.xpm");
}


void
on_reload_enter                     (GtkButton       *button,
                                        gpointer         user_data)
{
	toolbar.active = GTK_WIDGET (button);
	toolbar.prev = lookup_widget (GTK_WIDGET (button), "forward");
	toolbar.next = lookup_widget (GTK_WIDGET (button), "home");
	gtk_set_mouse_position (GTK_WIDGET (button), 250, 25);
	activate_pengudrive (__FUNCTION__ ,6, 4);
//	chg_pixmap (button, "image_reload", "aktualisieren_cursor.xpm");
}


void
on_reload_leave                        (GtkButton       *button,
					gpointer         user_data)
{
//	chg_pixmap (button, "image_reload", "aktualisieren.xpm");
}


void
on_reload_clicked                      (GtkButton       *button,
                                        gpointer         user_data)
{
	gtk_moz_embed_reload (GTK_MOZ_EMBED (lookup_widget (GTK_WIDGET (button),
							 "gecko")),
			     GTK_MOZ_EMBED_FLAG_RELOADBYPASSCACHE);
//	chg_pixmap (GTK_WIDGET (button), "image_reload",
//		    "aktualisieren_gedrueckt.xpm");	
}


void
on_reload_released                     (GtkButton       *button,
                                        gpointer         user_data)
{
//	chg_pixmap (GTK_WIDGET (button), "image_reload",
//		    "aktualisieren_cursor.xpm");
}


void
on_home_enter                       (GtkButton       *button,
                                        gpointer         user_data)
{
	toolbar.active = GTK_WIDGET (button);
	toolbar.prev = lookup_widget (GTK_WIDGET (button), "reload");
	toolbar.next = NULL;
	gtk_set_mouse_position (GTK_WIDGET (button), 300, 25);
	activate_pengudrive (__FUNCTION__ ,6, 5);
//	chg_pixmap (button, "image_home", "home_cursor.xpm");
}


void
on_home_leave                          (GtkButton       *button,
					gpointer         user_data)
{
//	chg_pixmap (button, "image_home", "home.xpm");
}


void
on_home_clicked                        (GtkButton       *button,
                                        gpointer         user_data)
{
	gchar *text;
	g_print ("%s:", __FUNCTION__);
	text = gtk_entry_get_text (lookup_widget (GTK_WIDGET (button),
						  "entry_setup_url"));

	gtk_moz_embed_load_url (GTK_MOZ_EMBED (lookup_widget (GTK_WIDGET (button),
							      "gecko")), 
				(text) ? text : "http://www.bmw.de");
	gtk_entry_set_text (lookup_widget (GTK_WIDGET (button), "combo_entry_URI_entry"), text);
//	chg_pixmap (GTK_WIDGET (button), "image_home",
//		    "home_gedrueckt.xpm");
}
				

void
on_home_released                       (GtkButton       *button,
                                        gpointer         user_data)
{
//	chg_pixmap (GTK_WIDGET (button), "image_home", "home_cursor.xpm");
}


gboolean
on_combo_entry_URI_entry_focus         (GtkWidget       *widget,
                                        GtkDirectionType  direction,
                                        gpointer         user_data)
{
	g_print ("URI 1\n");
	return FALSE;
}


gboolean
on_eventbox_toplevel_motion_notify_event
                                        (GtkWidget       *widget,
                                        GdkEventMotion  *event,
                                        gpointer         user_data)
{
	g_print ("mouse moved\n");
	return FALSE;
}


void
on_combo_entry_URI_entry_activate      (GtkEntry         *entry,
                                        gpointer         user_data)
{
	g_print ("%s: URI = %s\n", __FUNCTION__, gtk_entry_get_text (entry));
	gtk_widget_hide (combo_list);
	hide_tastatur (GTK_WIDGET (entry));
	gtk_moz_embed_load_url (GTK_MOZ_EMBED (lookup_widget (GTK_WIDGET (entry),
							      "gecko")), 
				gtk_entry_get_text (entry));
}


gboolean
on_combo_entry_URI_button_press_event  (GtkWidget       *widget,
                                        GdkEventButton  *event,
                                        gpointer         user_data)
{

  return FALSE;
}


void
on_combo_entry_URI_entry_changed       (GtkEditable     *editable,
                                        gpointer         user_data)
{
	gchar *in;
	
	in = gtk_editable_get_chars (editable, 0, -1);
	
	g_print ("%s: text = >%s<\n", __FUNCTION__, in);
	g_free (in);
}

gboolean
on_eventbox_toolbar_scroll_event       (GtkWidget       *widget,
                                        GdkEvent        *event,
                                        gpointer         user_data)
{
	guint button = event->scroll.direction;
	
	g_print ("%s: Button %d activate: ", __FUNCTION__, button);
	switch (button)
	{
		
	case GDK_SCROLL_UP: 
		if (toolbar.prev) 
		{
			g_signal_emit_by_name ((gpointer) toolbar.prev, 
					       "enter"); 
			g_print ("prev\n");
		}
		else g_print ("ignored (first!)\n");
		break;
		
	case GDK_SCROLL_DOWN: 
		if (toolbar.next) 
		{
			g_signal_emit_by_name ((gpointer) toolbar.next, 
					       "enter");
			g_print ("next\n");
		}
		else g_print ("ignored (lastst!)\n");
		break;
		
	default: g_print ("ignored\n");
	}
	
	return FALSE;
}

gboolean
on_eventbox_URI_scroll_event           (GtkWidget       *widget,
                                        GdkEvent        *event,
                                        gpointer         user_data)
{
	guint button = event->scroll.direction;
	
	on_eventbox_URI_list_scroll_event (lookup_widget (combo_list, 
							  "eventbox_URI_list"),
					   event, user_data);
	
	g_print ("%s\n", __FUNCTION__);
	switch (button)
	{
	case GDK_SCROLL_UP:
		break;
		
	case GDK_SCROLL_DOWN:
		break;
		
	default: g_print ("ignored\n");
	}
	
	return FALSE;
}

/* }}} */

/* {{{ Gecko Callbacks */

void
reload_clicked_cb                      (GtkButton       *button,
					gpointer         user_data)
{
	g_print ("%s: starting\n", __FUNCTION__);
	gtk_moz_embed_reload (GTK_MOZ_EMBED (lookup_widget (GTK_WIDGET (button),
							    "gecko")),
			      GTK_MOZ_EMBED_FLAG_RELOADBYPASSCACHE);
	g_print ("%s: finished\n", __FUNCTION__);
}


void
stop_clicked_cb                        (GtkButton       *button,
					gpointer         user_data)
{
	g_print("%s: starting\n",__FUNCTION__);
	gtk_moz_embed_stop_load(GTK_MOZ_EMBED(lookup_widget(GTK_WIDGET(button),"gecko")));
	g_print("%s: finished\n",__FUNCTION__);
}


/* }}} */

/* {{{ Setup Dialog */

voidon_button_setup_ok_enter            (GtkButton       *button,
                                        gpointer         user_data)
{

}

void
on_button_setup_ok_clicked             (GtkButton       *button,
                                        gpointer         user_data)
{
	char doit [1024];
	
	snprintf (doit, 1024, 
		  "/etc/init.d/ppp-update start \"%s\" \"%s\" \"%s\"\n",
		  gtk_entry_get_text (lookup_widget (GTK_WIDGET (button),
						     "entry_setup_user")),
		  gtk_entry_get_text (lookup_widget (GTK_WIDGET (button),
						     "entry_setup_pass")),
		  gtk_entry_get_text (lookup_widget (GTK_WIDGET (button),
						     "entry_setup_pin")));
	
	gtk_widget_hide (lookup_widget (GTK_WIDGET (button), "eventbox_setup"));

	hide_tastatur (GTK_WIDGET (button));
	gtk_widget_show (lookup_widget (GTK_WIDGET (button), "eventbox_gecko"));
	
	save_rc (PENGUZILLA_SETTINGS, GTK_WIDGET (button));
	
	g_print ("doit %s\n", doit);
	system (doit);
}

void
on_button_setup_cancel_enter        (GtkButton       *button,
                                        gpointer         user_data)
{

}

void
on_button_setup_cancel_clicked         (GtkButton       *button,
                                        gpointer         user_data)
{
	gtk_widget_hide (lookup_widget (GTK_WIDGET (button), "eventbox_setup"));
	hide_tastatur (GTK_WIDGET (button));
	gtk_widget_show (lookup_widget (GTK_WIDGET (button), "eventbox_gecko"));
}

/* }}} */

/* {{{ Generic Keyboard 1 Events */

gboolean
on_eventbox_tastatur1_enter_notify_event
                                        (GtkWidget       *widget,
                                        GdkEventCrossing *event,
                                        gpointer         user_data)
{
	return FALSE;
}


gboolean
on_eventbox_tastatur1_leave_notify_event
                                        (GtkWidget       *widget,
                                        GdkEventCrossing *event,
                                        gpointer         user_data)
{

	return FALSE;
}


gboolean
on_eventbox_tastatur1_scroll_event     (GtkWidget       *widget,
                                        GdkEvent        *event,
                                        gpointer         user_data)
{
	guint button = event->scroll.direction;
	
	g_print ("Button %d activate: ", button);
	switch (button)
	{
		
	case GDK_SCROLL_UP: 
		if (tastatur.prev) 
		{
			gtk_widget_grab_default (tastatur.prev);
			g_signal_emit_by_name ((gpointer) tastatur.prev,
					       "notify");
			g_print ("prev\n");
		}
		else g_print ("ignored (first!)\n");
		break;
		
	case GDK_SCROLL_DOWN: 
		if (tastatur.next) 
		{
			gtk_widget_grab_default (tastatur.next);
			g_signal_emit_by_name ((gpointer) tastatur.next, 
					       "notify");
			g_print ("next\n");
		}
		else g_print ("ignored (lastst!)\n");
		break;
		
	default: g_print ("ignored\n");
	}
	
	return FALSE;
}


gboolean
on_eventbox_tastatur1_button_press_event
                                        (GtkWidget       *widget,
                                        GdkEventButton  *event,
                                        gpointer         user_data)
{

	return FALSE;
}


void
on_eventbox_tastatur1_show             (GtkWidget       *widget,
                                        gpointer         user_data)
{
	
}

/* }}} */

/* {{{ Keyboard 1: normal letters */

void
on_tast_a_enter (GtkButton *button, gpointer user_data)
{
	tastatur.active = GTK_WIDGET (button);
	tastatur.prev = NULL;
	tastatur.next = lookup_widget (GTK_WIDGET (button), "tast_b");
	tastatur.px = 550;
	tastatur.py = 50;
	tastatur.nx = 570;
	tastatur.ny = 50;
gtk_widget_grab_default (tastatur.active);
	tastatur.pos = 0;
	activate_pengudrive (__FUNCTION__ ,39, tastatur.pos);
}

void
on_tast_a_clicked (GtkButton *button, gpointer user_data)
{
	add_char (tastatur.ref, "a", 1);
}

void
on_tast_b_enter (GtkButton *button, gpointer user_data)
{
	tastatur.active = GTK_WIDGET (button);
	tastatur.prev = lookup_widget (GTK_WIDGET (button), "tast_a");
	tastatur.next = lookup_widget (GTK_WIDGET (button), "tast_c");
	tastatur.px = 550;
	tastatur.py = 50;
	tastatur.nx = 590;
	tastatur.ny = 50;
gtk_widget_grab_default (tastatur.active);
	tastatur.pos = 1;
	activate_pengudrive (__FUNCTION__ ,39, tastatur.pos);
}

void
on_tast_b_clicked (GtkButton *button, gpointer user_data)
{
	add_char (tastatur.ref, "b", 1);
}

void
on_tast_c_enter (GtkButton *button, gpointer user_data)
{
	tastatur.active = GTK_WIDGET (button);
	tastatur.prev = lookup_widget (GTK_WIDGET (button), "tast_b");
	tastatur.next = lookup_widget (GTK_WIDGET (button), "tast_d");
	tastatur.px = 570;
	tastatur.py = 50;
	tastatur.nx = 610;
	tastatur.ny = 50;
gtk_widget_grab_default (tastatur.active);
	tastatur.pos = 2;
	activate_pengudrive (__FUNCTION__ ,39, tastatur.pos);
}

void
on_tast_c_clicked (GtkButton *button, gpointer user_data)
{
	add_char (tastatur.ref, "c", 1);
}

void
on_tast_d_enter (GtkButton *button, gpointer user_data)
{
	tastatur.active = GTK_WIDGET (button);
	tastatur.prev = lookup_widget (GTK_WIDGET (button), "tast_c");
	tastatur.next = lookup_widget (GTK_WIDGET (button), "tast_e");
	tastatur.px = 590;
	tastatur.py = 50;
	tastatur.nx = 630;
	tastatur.ny = 50;
gtk_widget_grab_default (tastatur.active);
	tastatur.pos = 3;
	activate_pengudrive (__FUNCTION__ ,39, tastatur.pos);
}

void
on_tast_d_clicked (GtkButton *button, gpointer user_data)
{
	add_char (tastatur.ref, "d", 1);
}

void
on_tast_e_enter (GtkButton *button, gpointer user_data)
{
	tastatur.active = GTK_WIDGET (button);
	tastatur.prev = lookup_widget (GTK_WIDGET (button), "tast_d");
	tastatur.next = lookup_widget (GTK_WIDGET (button), "tast_f");
	tastatur.px = 610;
	tastatur.py = 50;
	tastatur.nx = 550;
	tastatur.ny = 70;
gtk_widget_grab_default (tastatur.active);
	tastatur.pos = 4;
	activate_pengudrive (__FUNCTION__ ,39, tastatur.pos);
}

void
on_tast_e_clicked (GtkButton *button, gpointer user_data)
{
	add_char (tastatur.ref, "e", 1);
}

void
on_tast_f_enter (GtkButton *button, gpointer user_data)
{
	tastatur.active = GTK_WIDGET (button);
	tastatur.prev = lookup_widget (GTK_WIDGET (button), "tast_e");
	tastatur.next = lookup_widget (GTK_WIDGET (button), "tast_g");
	tastatur.px = 630;
	tastatur.py = 50;
	tastatur.nx = 570;
	tastatur.ny = 70;
gtk_widget_grab_default (tastatur.active);
	tastatur.pos = 5;
	activate_pengudrive (__FUNCTION__ ,39, tastatur.pos);
}

void
on_tast_f_clicked (GtkButton *button, gpointer user_data)
{
	add_char (tastatur.ref, "f", 1);
}

void
on_tast_g_enter (GtkButton *button, gpointer user_data)
{
	tastatur.active = GTK_WIDGET (button);
	tastatur.prev = lookup_widget (GTK_WIDGET (button), "tast_f");
	tastatur.next = lookup_widget (GTK_WIDGET (button), "tast_h");
	tastatur.px = 550;
	tastatur.py = 70;
	tastatur.nx = 590;
	tastatur.ny = 70;
gtk_widget_grab_default (tastatur.active);
	tastatur.pos = 6;
	activate_pengudrive (__FUNCTION__ ,39, tastatur.pos);
}

void
on_tast_g_clicked (GtkButton *button, gpointer user_data)
{
	add_char (tastatur.ref, "g", 1);
}

void
on_tast_h_enter (GtkButton *button, gpointer user_data)
{
	tastatur.active = GTK_WIDGET (button);
	tastatur.prev = lookup_widget (GTK_WIDGET (button), "tast_g");
	tastatur.next = lookup_widget (GTK_WIDGET (button), "tast_i");
	tastatur.px = 570;
	tastatur.py = 70;
	tastatur.nx = 610;
	tastatur.ny = 70;
gtk_widget_grab_default (tastatur.active);
	tastatur.pos = 7;
	activate_pengudrive (__FUNCTION__ ,39, tastatur.pos);
}

void
on_tast_h_clicked (GtkButton *button, gpointer user_data)
{
	add_char (tastatur.ref, "h", 1);
}

void
on_tast_i_enter (GtkButton *button, gpointer user_data)
{
	tastatur.active = GTK_WIDGET (button);
	tastatur.prev = lookup_widget (GTK_WIDGET (button), "tast_h");
	tastatur.next = lookup_widget (GTK_WIDGET (button), "tast_j");
	tastatur.px = 590;
	tastatur.py = 70;
	tastatur.nx = 630;
	tastatur.ny = 70;
gtk_widget_grab_default (tastatur.active);
	tastatur.pos = 8;
	activate_pengudrive (__FUNCTION__ ,39, tastatur.pos);
}

void
on_tast_i_clicked (GtkButton *button, gpointer user_data)
{
	add_char (tastatur.ref, "i", 1);
}

void
on_tast_j_enter (GtkButton *button, gpointer user_data)
{
	tastatur.active = GTK_WIDGET (button);
	tastatur.prev = lookup_widget (GTK_WIDGET (button), "tast_i");
	tastatur.next = lookup_widget (GTK_WIDGET (button), "tast_k");
	tastatur.px = 610;
	tastatur.py = 70;
	tastatur.nx = 550;
	tastatur.ny = 90;
gtk_widget_grab_default (tastatur.active);
	tastatur.pos = 9;
	activate_pengudrive (__FUNCTION__ ,39, tastatur.pos);
}

void
on_tast_j_clicked (GtkButton *button, gpointer user_data)
{
	add_char (tastatur.ref, "j", 1);
}

void
on_tast_k_enter (GtkButton *button, gpointer user_data)
{
	tastatur.active = GTK_WIDGET (button);
	tastatur.prev = lookup_widget (GTK_WIDGET (button), "tast_j");
	tastatur.next = lookup_widget (GTK_WIDGET (button), "tast_l");
	tastatur.px = 630;
	tastatur.py = 70;
	tastatur.nx = 570;
	tastatur.ny = 90;
gtk_widget_grab_default (tastatur.active);
	tastatur.pos = 10;
	activate_pengudrive (__FUNCTION__ ,39, tastatur.pos);
}

void
on_tast_k_clicked (GtkButton *button, gpointer user_data)
{
	add_char (tastatur.ref, "k", 1);
}

void
on_tast_l_enter (GtkButton *button, gpointer user_data)
{
	tastatur.active = GTK_WIDGET (button);
	tastatur.prev = lookup_widget (GTK_WIDGET (button), "tast_k");
	tastatur.next = lookup_widget (GTK_WIDGET (button), "tast_m");
	tastatur.px = 550;
	tastatur.py = 90;
	tastatur.nx = 590;
	tastatur.ny = 90;
gtk_widget_grab_default (tastatur.active);
	tastatur.pos = 11;
	activate_pengudrive (__FUNCTION__ ,39, tastatur.pos);
}

void
on_tast_l_clicked (GtkButton *button, gpointer user_data)
{
	add_char (tastatur.ref, "l", 1);
}

void
on_tast_m_enter (GtkButton *button, gpointer user_data)
{
	tastatur.active = GTK_WIDGET (button);
	tastatur.prev = lookup_widget (GTK_WIDGET (button), "tast_l");
	tastatur.next = lookup_widget (GTK_WIDGET (button), "tast_n");
	tastatur.px = 570;
	tastatur.py = 90;
	tastatur.nx = 610;
	tastatur.ny = 90;
gtk_widget_grab_default (tastatur.active);
	tastatur.pos = 12;
	activate_pengudrive (__FUNCTION__ ,39, tastatur.pos);
}

void
on_tast_m_clicked (GtkButton *button, gpointer user_data)
{
	add_char (tastatur.ref, "m", 1);
}

void
on_tast_n_enter (GtkButton *button, gpointer user_data)
{
	tastatur.active = GTK_WIDGET (button);
	tastatur.prev = lookup_widget (GTK_WIDGET (button), "tast_m");
	tastatur.next = lookup_widget (GTK_WIDGET (button), "tast_o");
	tastatur.px = 590;
	tastatur.py = 90;
	tastatur.nx = 630;
	tastatur.ny = 90;
gtk_widget_grab_default (tastatur.active);
	tastatur.pos = 13;
	activate_pengudrive (__FUNCTION__ ,39, tastatur.pos);
}

void
on_tast_n_clicked (GtkButton *button, gpointer user_data)
{
	add_char (tastatur.ref, "n", 1);
}

void
on_tast_o_enter (GtkButton *button, gpointer user_data)
{
	tastatur.active = GTK_WIDGET (button);
	tastatur.prev = lookup_widget (GTK_WIDGET (button), "tast_n");
	tastatur.next = lookup_widget (GTK_WIDGET (button), "tast_p");
	tastatur.px = 610;
	tastatur.py = 90;
	tastatur.nx = 550;
	tastatur.ny = 110;
gtk_widget_grab_default (tastatur.active);
	tastatur.pos = 14;
	activate_pengudrive (__FUNCTION__ ,39, tastatur.pos);
}

void
on_tast_o_clicked (GtkButton *button, gpointer user_data)
{
	add_char (tastatur.ref, "o", 1);
}

void
on_tast_p_enter (GtkButton *button, gpointer user_data)
{
	tastatur.active = GTK_WIDGET (button);
	tastatur.prev = lookup_widget (GTK_WIDGET (button), "tast_o");
	tastatur.next = lookup_widget (GTK_WIDGET (button), "tast_q");
	tastatur.px = 630;
	tastatur.py = 90;
	tastatur.nx = 570;
	tastatur.ny = 110;
gtk_widget_grab_default (tastatur.active);
	tastatur.pos = 15;
	activate_pengudrive (__FUNCTION__ ,39, tastatur.pos);
}

void
on_tast_p_clicked (GtkButton *button, gpointer user_data)
{
	add_char (tastatur.ref, "p", 1);
}

void
on_tast_q_enter (GtkButton *button, gpointer user_data)
{
	tastatur.active = GTK_WIDGET (button);
	tastatur.prev = lookup_widget (GTK_WIDGET (button), "tast_p");
	tastatur.next = lookup_widget (GTK_WIDGET (button), "tast_r");
	tastatur.px = 550;
	tastatur.py = 110;
	tastatur.nx = 590;
	tastatur.ny = 110;
gtk_widget_grab_default (tastatur.active);
	tastatur.pos = 16;
	activate_pengudrive (__FUNCTION__ ,39, tastatur.pos);
}

void
on_tast_q_clicked (GtkButton *button, gpointer user_data)
{
	add_char (tastatur.ref, "q", 1);
}

void
on_tast_r_enter (GtkButton *button, gpointer user_data)
{
	tastatur.active = GTK_WIDGET (button);
	tastatur.prev = lookup_widget (GTK_WIDGET (button), "tast_q");
	tastatur.next = lookup_widget (GTK_WIDGET (button), "tast_s");
	tastatur.px = 570;
	tastatur.py = 110;
	tastatur.nx = 610;
	tastatur.ny = 110;
gtk_widget_grab_default (tastatur.active);
	tastatur.pos = 17;
	activate_pengudrive (__FUNCTION__ ,39, tastatur.pos);
}

void
on_tast_r_clicked (GtkButton *button, gpointer user_data)
{
	add_char (tastatur.ref, "r", 1);
}

void
on_tast_s_enter (GtkButton *button, gpointer user_data)
{
	tastatur.active = GTK_WIDGET (button);
	tastatur.prev = lookup_widget (GTK_WIDGET (button), "tast_r");
	tastatur.next = lookup_widget (GTK_WIDGET (button), "tast_t");
	tastatur.px = 590;
	tastatur.py = 110;
	tastatur.nx = 630;
	tastatur.ny = 110;
gtk_widget_grab_default (tastatur.active);
	tastatur.pos = 18;
	activate_pengudrive (__FUNCTION__ ,39, tastatur.pos);
}

void
on_tast_s_clicked (GtkButton *button, gpointer user_data)
{
	add_char (tastatur.ref, "s", 1);
}

void
on_tast_t_enter (GtkButton *button, gpointer user_data)
{
	tastatur.active = GTK_WIDGET (button);
	tastatur.prev = lookup_widget (GTK_WIDGET (button), "tast_s");
	tastatur.next = lookup_widget (GTK_WIDGET (button), "tast_u");
	tastatur.px = 610;
	tastatur.py = 110;
	tastatur.nx = 550;
	tastatur.ny = 130;
gtk_widget_grab_default (tastatur.active);
	tastatur.pos = 19;
	activate_pengudrive (__FUNCTION__ ,39, tastatur.pos);
}

void
on_tast_t_clicked (GtkButton *button, gpointer user_data)
{
	add_char (tastatur.ref, "t", 1);
}

void
on_tast_u_enter (GtkButton *button, gpointer user_data)
{
	tastatur.active = GTK_WIDGET (button);
	tastatur.prev = lookup_widget (GTK_WIDGET (button), "tast_t");
	tastatur.next = lookup_widget (GTK_WIDGET (button), "tast_v");
	tastatur.px = 630;
	tastatur.py = 110;
	tastatur.nx = 570;
	tastatur.ny = 130;
gtk_widget_grab_default (tastatur.active);
	tastatur.pos = 20;
	activate_pengudrive (__FUNCTION__ ,39, tastatur.pos);
}

void
on_tast_u_clicked (GtkButton *button, gpointer user_data)
{
	add_char (tastatur.ref, "u", 1);
}

void
on_tast_v_enter (GtkButton *button, gpointer user_data)
{
	tastatur.active = GTK_WIDGET (button);
	tastatur.prev = lookup_widget (GTK_WIDGET (button), "tast_u");
	tastatur.next = lookup_widget (GTK_WIDGET (button), "tast_w");
	tastatur.px = 550;
	tastatur.py = 130;
	tastatur.nx = 590;
	tastatur.ny = 130;
gtk_widget_grab_default (tastatur.active);
	tastatur.pos = 21;
	activate_pengudrive (__FUNCTION__ ,39, tastatur.pos);
}

void
on_tast_v_clicked (GtkButton *button, gpointer user_data)
{
	add_char (tastatur.ref, "v", 1);
}

void
on_tast_w_enter (GtkButton *button, gpointer user_data)
{
	tastatur.active = GTK_WIDGET (button);
	tastatur.prev = lookup_widget (GTK_WIDGET (button), "tast_v");
	tastatur.next = lookup_widget (GTK_WIDGET (button), "tast_x");
	tastatur.px = 570;
	tastatur.py = 130;
	tastatur.nx = 610;
	tastatur.ny = 130;
gtk_widget_grab_default (tastatur.active);
	tastatur.pos = 22;
	activate_pengudrive (__FUNCTION__ ,39, tastatur.pos);
}

void
on_tast_w_clicked (GtkButton *button, gpointer user_data)
{
	add_char (tastatur.ref, "w", 1);
}

void
on_tast_x_enter (GtkButton *button, gpointer user_data)
{
	tastatur.active = GTK_WIDGET (button);
	tastatur.prev = lookup_widget (GTK_WIDGET (button), "tast_w");
	tastatur.next = lookup_widget (GTK_WIDGET (button), "tast_y");
	tastatur.px = 590;
	tastatur.py = 130;
	tastatur.nx = 630;
	tastatur.ny = 130;
gtk_widget_grab_default (tastatur.active);
	tastatur.pos = 23;
	activate_pengudrive (__FUNCTION__ ,39, tastatur.pos);
}

void
on_tast_x_clicked (GtkButton *button, gpointer user_data)
{
	add_char (tastatur.ref, "x", 1);
}

void
on_tast_y_enter (GtkButton *button, gpointer user_data)
{
	tastatur.active = GTK_WIDGET (button);
	tastatur.prev = lookup_widget (GTK_WIDGET (button), "tast_x");
	tastatur.next = lookup_widget (GTK_WIDGET (button), "tast_z");
	tastatur.px = 610;
	tastatur.py = 130;
	tastatur.nx = 550;
	tastatur.ny = 150;
gtk_widget_grab_default (tastatur.active);
	tastatur.pos = 24;
	activate_pengudrive (__FUNCTION__ ,39, tastatur.pos);
}

void
on_tast_y_clicked (GtkButton *button, gpointer user_data)
{
	add_char (tastatur.ref, "y", 1);
}

void
on_tast_z_enter (GtkButton *button, gpointer user_data)
{
	tastatur.active = GTK_WIDGET (button);
	tastatur.prev = lookup_widget (GTK_WIDGET (button), "tast_y");
	tastatur.next = lookup_widget (GTK_WIDGET (button), "tast_leer1");
	tastatur.px = 630;
	tastatur.py = 130;
	tastatur.nx = 570;
	tastatur.ny = 150;
gtk_widget_grab_default (tastatur.active);
	tastatur.pos = 25;
	activate_pengudrive (__FUNCTION__ ,39, tastatur.pos);
}

void
on_tast_z_clicked (GtkButton *button, gpointer user_data)
{
	add_char (tastatur.ref, "z", 1);
}

void
on_tast_leer1_enter (GtkButton *button, gpointer user_data)
{
	tastatur.active = GTK_WIDGET (button);
	tastatur.prev = lookup_widget (GTK_WIDGET (button), "tast_z");
	tastatur.next = lookup_widget (GTK_WIDGET (button), "tast_slash1");
	tastatur.px = 550;
	tastatur.py = 150;
	tastatur.nx = 590;
	tastatur.ny = 150;
gtk_widget_grab_default (tastatur.active);
	tastatur.pos = 26;
	activate_pengudrive (__FUNCTION__ ,39, tastatur.pos);
}

void
on_tast_leer1_clicked (GtkButton *button, gpointer user_data)
{
	add_char (tastatur.ref, " ", 1);
}

void
on_tast_slash1_enter (GtkButton *button, gpointer user_data)
{
	tastatur.active = GTK_WIDGET (button);
	tastatur.prev = lookup_widget (GTK_WIDGET (button), "tast_leer1");
	tastatur.next = lookup_widget (GTK_WIDGET (button), "tast_dot1");
	tastatur.px = 570;
	tastatur.py = 150;
	tastatur.nx = 610;
	tastatur.ny = 150;
gtk_widget_grab_default (tastatur.active);
	tastatur.pos = 27;
	activate_pengudrive (__FUNCTION__ ,39, tastatur.pos);
}

void
on_tast_slash1_clicked (GtkButton *button, gpointer user_data)
{
	add_char (tastatur.ref, "/", 1);
}

void
on_tast_dot1_enter (GtkButton *button, gpointer user_data)
{
	tastatur.active = GTK_WIDGET (button);
	tastatur.prev = lookup_widget (GTK_WIDGET (button), "tast_slash1");
	tastatur.next = lookup_widget (GTK_WIDGET (button), "tast_vor1");
	tastatur.px = 590;
	tastatur.py = 150;
	tastatur.nx = 630;
	tastatur.ny = 150;
gtk_widget_grab_default (tastatur.active);
	tastatur.pos = 28;
	activate_pengudrive (__FUNCTION__ ,39, tastatur.pos);
}

void
on_tast_dot1_clicked (GtkButton *button, gpointer user_data)
{
	add_char (tastatur.ref, ".", 1);
}

void
on_tast_vor1_enter                   (GtkButton       *button,
                                        gpointer         user_data)
{
	GtkWidget *image_setup;
	tastatur.active = GTK_WIDGET (button);
	tastatur.prev = lookup_widget (GTK_WIDGET (button), "tast_dot1");
	tastatur.next = lookup_widget (GTK_WIDGET (button), "tast_www");
	tastatur.px = 610;
	tastatur.py = 150;
	tastatur.nx = 560;
	tastatur.ny = 170;
gtk_widget_grab_default (tastatur.active);
	tastatur.pos = 29;
	activate_pengudrive (__FUNCTION__ ,39, tastatur.pos);
}

void
on_tast_vor1_clicked                    (GtkButton       *button,
                                        gpointer         user_data)
{
	g_print ("Change to 0\n");

	activate_pengudrive (__FUNCTION__ ,43, tastatur.pos);
	gtk_widget_hide (lookup_widget (GTK_WIDGET (button), 
					"eventbox_tastatur1"));
       	gtk_widget_show (lookup_widget (GTK_WIDGET (button), 
					"eventbox_tastatur"));
	g_signal_emit_by_name ((gpointer) lookup_widget (GTK_WIDGET (button),
							 "tast_vor"),
			       "notify");
	keyboard1 = FALSE;
}


void
on_tast_0_enter (GtkButton *button, gpointer user_data)
{
	tastatur.active = GTK_WIDGET (button);
	tastatur.prev = lookup_widget (GTK_WIDGET (button), "tast_vor");
	tastatur.next = lookup_widget (GTK_WIDGET (button), "tast_1");
	tastatur.px = 630;
	tastatur.py = 150;
	tastatur.nx = 570;
	tastatur.ny = 170;
gtk_widget_grab_default (tastatur.active);
	tastatur.pos = 30;
	activate_pengudrive (__FUNCTION__ ,39, tastatur.pos);
}

void
on_tast_0_clicked (GtkButton *button, gpointer user_data)
{
	add_char (tastatur.ref, "0", 1);
}

void
on_tast_1_enter (GtkButton *button, gpointer user_data)
{
	tastatur.active = GTK_WIDGET (button);
	tastatur.prev = lookup_widget (GTK_WIDGET (button), "tast_0");
	tastatur.next = lookup_widget (GTK_WIDGET (button), "tast_2");
	tastatur.px = 550;
	tastatur.py = 170;
	tastatur.nx = 590;
	tastatur.ny = 170;
gtk_widget_grab_default (tastatur.active);
	tastatur.pos = 31;
	activate_pengudrive (__FUNCTION__ ,43, tastatur.pos);
}

void
on_tast_1_clicked (GtkButton *button, gpointer user_data)
{
	add_char (tastatur.ref, "1", 1);
}

void
on_tast_2_enter (GtkButton *button, gpointer user_data)
{
	tastatur.active = GTK_WIDGET (button);
	tastatur.prev = lookup_widget (GTK_WIDGET (button), "tast_1");
	tastatur.next = lookup_widget (GTK_WIDGET (button), "tast_3");
	tastatur.px = 570;
	tastatur.py = 170;
	tastatur.nx = 610;
	tastatur.ny = 170;
gtk_widget_grab_default (tastatur.active);
	tastatur.pos = 32;
	activate_pengudrive (__FUNCTION__ ,43, tastatur.pos);
}

void
on_tast_2_clicked (GtkButton *button, gpointer user_data)
{
	add_char (tastatur.ref, "2", 1);
}

void
on_tast_3_enter (GtkButton *button, gpointer user_data)
{
	tastatur.active = GTK_WIDGET (button);
	tastatur.prev = lookup_widget (GTK_WIDGET (button), "tast_2");
	tastatur.next = lookup_widget (GTK_WIDGET (button), "tast_4");
	tastatur.px = 590;
	tastatur.py = 170;
	tastatur.nx = 630;
	tastatur.ny = 170;
gtk_widget_grab_default (tastatur.active);
	tastatur.pos = 33;
	activate_pengudrive (__FUNCTION__ ,43, tastatur.pos);
}

void
on_tast_3_clicked (GtkButton *button, gpointer user_data)
{
	add_char (tastatur.ref, "3", 1);
}

void
on_tast_4_enter (GtkButton *button, gpointer user_data)
{
	tastatur.active = GTK_WIDGET (button);
	tastatur.prev = lookup_widget (GTK_WIDGET (button), "tast_3");
	tastatur.next = lookup_widget (GTK_WIDGET (button), "tast_5");
	tastatur.px = 610;
	tastatur.py = 170;
	tastatur.nx = 550;
	tastatur.ny = 190;
gtk_widget_grab_default (tastatur.active);
	tastatur.pos = 34;
	activate_pengudrive (__FUNCTION__ ,43, tastatur.pos);
}

void
on_tast_4_clicked (GtkButton *button, gpointer user_data)
{
	add_char (tastatur.ref, "4", 1);
}

void
on_tast_5_enter (GtkButton *button, gpointer user_data)
{
	tastatur.active = GTK_WIDGET (button);
	tastatur.prev = lookup_widget (GTK_WIDGET (button), "tast_4");
	tastatur.next = lookup_widget (GTK_WIDGET (button), "tast_6");
	tastatur.px = 630;
	tastatur.py = 170;
	tastatur.nx = 570;
	tastatur.ny = 190;
gtk_widget_grab_default (tastatur.active);
	tastatur.pos = 35;
	activate_pengudrive (__FUNCTION__ ,43, tastatur.pos);
}

void
on_tast_5_clicked (GtkButton *button, gpointer user_data)
{
	add_char (tastatur.ref, "5", 1);
}

void
on_tast_6_enter (GtkButton *button, gpointer user_data)
{
	tastatur.active = GTK_WIDGET (button);
	tastatur.prev = lookup_widget (GTK_WIDGET (button), "tast_5");
	tastatur.next = lookup_widget (GTK_WIDGET (button), "tast_7");
	tastatur.px = 550;
	tastatur.py = 190;
	tastatur.nx = 590;
	tastatur.ny = 190;
gtk_widget_grab_default (tastatur.active);
	tastatur.pos = 36;
	activate_pengudrive (__FUNCTION__ ,43, tastatur.pos);
}

void
on_tast_6_clicked (GtkButton *button, gpointer user_data)
{
	add_char (tastatur.ref, "6", 1);
}

void
on_tast_7_enter (GtkButton *button, gpointer user_data)
{
	tastatur.active = GTK_WIDGET (button);
	tastatur.prev = lookup_widget (GTK_WIDGET (button), "tast_6");
	tastatur.next = lookup_widget (GTK_WIDGET (button), "tast_8");
	tastatur.px = 570;
	tastatur.py = 190;
	tastatur.nx = 610;
	tastatur.ny = 190;
gtk_widget_grab_default (tastatur.active);
	tastatur.pos = 37;
	activate_pengudrive (__FUNCTION__ ,43, tastatur.pos);
}

void
on_tast_7_clicked (GtkButton *button, gpointer user_data)
{
	add_char (tastatur.ref, "7", 1);
}

void
on_tast_8_enter (GtkButton *button, gpointer user_data)
{
	tastatur.active = GTK_WIDGET (button);
	tastatur.prev = lookup_widget (GTK_WIDGET (button), "tast_7");
	tastatur.next = lookup_widget (GTK_WIDGET (button), "tast_9");
	tastatur.px = 590;
	tastatur.py = 190;
	tastatur.nx = 630;
	tastatur.ny = 190;
gtk_widget_grab_default (tastatur.active);
	tastatur.pos = 38;
	activate_pengudrive (__FUNCTION__ ,43, tastatur.pos);
}

void
on_tast_8_clicked (GtkButton *button, gpointer user_data)
{
	add_char (tastatur.ref, "8", 1);
}

void
on_tast_9_enter (GtkButton *button, gpointer user_data)
{
	tastatur.active = GTK_WIDGET (button);
	tastatur.prev = lookup_widget (GTK_WIDGET (button), "tast_8");
	tastatur.next = lookup_widget (GTK_WIDGET (button), "tast_enter");
	tastatur.px = 610;
	tastatur.py = 190;
	tastatur.nx = 541;
	tastatur.ny = 230;
	gtk_widget_grab_default (tastatur.active);
	tastatur.pos = 39;
	activate_pengudrive (__FUNCTION__ ,43, tastatur.pos);
}

void
on_tast_9_clicked (GtkButton *button, gpointer user_data)
{
	add_char (tastatur.ref, "9", 1);
}

void
on_tast_enter1_enter                (GtkButton       *button,
                                        gpointer         user_data)
{
	tastatur.active = GTK_WIDGET (button);
	tastatur.prev = lookup_widget (GTK_WIDGET (button), "tast_dash");
	tastatur.next = lookup_widget (GTK_WIDGET (button), "tast_esc1");
	tastatur.px = 630;
	tastatur.py = 190;
	tastatur.nx = 574;
	tastatur.ny = 230;
	gtk_widget_grab_default (tastatur.active);
	tastatur.pos = 36;
	activate_pengudrive (__FUNCTION__ ,39, tastatur.pos);
}

void
on_tast_enter1_clicked                  (GtkButton	*button,
                                        gpointer	user_data)
{
	hide_tastatur (GTK_WIDGET (button));
	disable_pengudrive (__FUNCTION__);
	tastatur.fix_mouse = FALSE;

	atk_action_do_action (tastatur.ref, 0);
}

void
on_tast_esc1_enter                 (GtkButton       *button,
                                        gpointer         user_data)
{
	tastatur.active = GTK_WIDGET (button);
	tastatur.prev = lookup_widget (GTK_WIDGET (button), "tast_enter1");
	tastatur.next = lookup_widget (GTK_WIDGET (button), "tast_back1");
	tastatur.px = 541;
	tastatur.py = 230;
	tastatur.nx = 607;
	tastatur.ny = 230;
	gtk_widget_grab_default (tastatur.active);
	tastatur.pos = 37;
	activate_pengudrive (__FUNCTION__ ,39, tastatur.pos);
}

void
on_tast_esc1_clicked                  (GtkButton       *button,
                                        gpointer         user_data)
{
	gint l;
	gchar *text;
	
	hide_tastatur (GTK_WIDGET (button));
	
	if (ATK_IS_EDITABLE_TEXT (tastatur.ref))
	{
		text = atk_text_get_text (ATK_TEXT (tastatur.ref), 0, -1);
		if (text)
		{
			l = strlen (text);
			atk_editable_text_delete_text (ATK_EDITABLE_TEXT (tastatur.ref), 
						       0, l);
		}
	}
	disable_pengudrive (__FUNCTION__);
	tastatur.fix_mouse = FALSE;
}


/* }}} */

void
on_combo_entry_URI_set_focus_child     (GtkContainer    *container,
                                        GtkWidget       *widget,
                                        gpointer         user_data)
{
	AtkObject	*aobj;
	
	if (!widget) return;
	g_print ("URI\n");
	aobj = atk_gobject_accessible_for_object (G_OBJECT (lookup_widget (widget, 
									   "combo_entry_URI_entry")));
	atk_focus_tracker_notify (aobj);
}




gboolean
on_eventbox_toolbar_enter_notify_event (GtkWidget       *widget,
                                        GdkEventCrossing *event,
                                        gpointer         user_data)
{
	return FALSE;
}


gboolean
on_eventbox_toolbar_leave_notify_event (GtkWidget       *widget,
                                        GdkEventCrossing *event,
                                        gpointer         user_data)
{
	disable_pengudrive (__FUNCTION__);
	return FALSE;
}


void
on_entry_setup_url_enter            (GtkEntry        *entry,
                                        gpointer         user_data)
{

}


void
on_entry_setup_pin_enter            (GtkEntry        *entry,
                                        gpointer         user_data)
{

}


void
on_entry_setup_dns_enter            (GtkEntry        *entry,
                                        gpointer         user_data)
{

}


void
on_entry_setup_user_enter           (GtkEntry        *entry,
                                        gpointer         user_data)
{

}


void
on_entry_setup_pass_enter           (GtkEntry        *entry,
                                        gpointer         user_data)
{

}


void
on_entry_setup_number_enter         (GtkEntry        *entry,
                                        gpointer         user_data)
{

}


void
on_entry_setup_ipaddress_enter      (GtkEntry        *entry,
                                        gpointer         user_data)
{

}


void
on_entry_setup_netmask_enter        (GtkEntry        *entry,
                                        gpointer         user_data)
{

}


void
on_entry_setup_gateway_enter        (GtkEntry        *entry,
                                        gpointer         user_data)
{

}


void
on_tast_back_enter                  (GtkButton       *button,
                                        gpointer         user_data)
{
	tastatur.active = GTK_WIDGET (button);
	tastatur.prev = lookup_widget (GTK_WIDGET (button), "tast_esc");
	tastatur.next = NULL;
	tastatur.px = 574;
	tastatur.py = 230;
	tastatur.nx = 607;
	tastatur.ny = 230;
gtk_widget_grab_default (tastatur.active);
	tastatur.pos = 42;
	activate_pengudrive (__FUNCTION__ ,43, tastatur.pos);
}


void
on_tast_back_clicked                   (GtkButton       *button,
                                        gpointer         user_data)
{
	guint pos = 0;
	AtkObject *aobj;
	
	aobj = ATK_OBJECT (tastatur.ref);
	g_print ("%s\n", __FUNCTION__);
	if (ATK_IS_EDITABLE_TEXT (aobj))
	{
		pos = atk_text_get_caret_offset (ATK_TEXT (aobj));
		g_print ("pos before: %d\n", pos);
		atk_editable_text_delete_text (ATK_EDITABLE_TEXT (aobj), 
					       pos - 1, pos + 1);
	}
}


void
on_tast_back1_enter                 (GtkButton       *button,
                                        gpointer         user_data)
{
	tastatur.active = GTK_WIDGET (button);
	tastatur.prev = lookup_widget (GTK_WIDGET (button), "tast_esc1");
	tastatur.next = NULL;
	tastatur.px = 574;
	tastatur.py = 230;
	tastatur.nx = 607;
	tastatur.ny = 230;
gtk_widget_grab_default (tastatur.active);
	tastatur.pos = 38;
	activate_pengudrive (__FUNCTION__ ,39, tastatur.pos);
}


void
on_tast_back1_clicked                  (GtkButton       *button,
                                        gpointer         user_data)
{
	guint pos = 0;
	AtkObject *aobj;
	
	aobj = ATK_OBJECT (tastatur.ref);
	g_print ("%s\n", __FUNCTION__);
	if (ATK_IS_EDITABLE_TEXT (aobj))
	{
		pos = atk_text_get_caret_offset (ATK_TEXT (aobj));
		g_print ("pos before: %d\n", pos);
		atk_editable_text_delete_text (ATK_EDITABLE_TEXT (aobj), 
					       pos - 1, pos + 1);
	}
}


gboolean
on_eventbox_toolbar_button_press_event (GtkWidget       *widget,
                                        GdkEventButton  *event,
                                        gpointer         user_data)
{
	
  return FALSE;
}


gboolean
on_eventbox_gecko_scroll_event         (GtkWidget       *widget,
                                        GdkEvent        *event,
                                        gpointer         user_data)
{
	GtkAdjustment *vadjustment;
	gfloat vvalue;
	guint button = event->scroll.direction;

	vadjustment = gtk_viewport_get_vadjustment (GTK_VIEWPORT (lookup_widget (GTK_WIDGET(widget),
										 "viewport1")));
	
	
	switch (button)
	{
		
	case GDK_SCROLL_UP:
		g_print ("%s: up\n", __FUNCTION__);
//		vvalue = vadjustment->value - 5;
		if (vvalue < 0) vvalue = 0;
		break;
		
	case GDK_SCROLL_DOWN: 
		g_print ("%s: down\n", __FUNCTION__);
//		vvalue = vadjustment->value + 5;
		if (vvalue > vadjustment->upper - vadjustment->page_size) 
		    vvalue = vadjustment->upper - vadjustment->page_size;
		break;
		
	default: g_print ("%s; Button ignored\n", __FUNCTION__);
	}

	gtk_adjustment_set_value(vadjustment, vvalue);
	gtk_viewport_set_vadjustment (GTK_VIEWPORT (lookup_widget (GTK_WIDGET(widget),
								   "viewport1")), 
				      vadjustment);
	
	return TRUE;
}


void
on_button_dhcp_enter                (GtkButton       *button,
                                        gpointer         user_data)
{

}


void
on_button_dhcp_clicked                 (GtkButton       *button,
                                        gpointer         user_data)
{

}


gboolean
on_eventbox_toolbar_motion_notify_event
                                        (GtkWidget       *widget,
                                        GdkEventMotion  *event,
                                        gpointer         user_data)
{
	g_print ("x: %f\ny: %f\n", event->x_root, event->y_root);
	return FALSE;
}


void
on_combo_entry_URI_entry_edit_done     (GtkCellEditable *celleditable,
                                        gpointer         user_data)
{
	g_print ("%s: done\n", __FUNCTION__);
}


void
on_tast_A_leave                        (GtkButton       *button,
                                        gpointer         user_data)
{

}

void
on_tast_B_leave                        (GtkButton       *button,
                                        gpointer         user_data)
{

}

void
on_tast_C_leave                        (GtkButton       *button,
                                        gpointer         user_data)
{

}


void
on_tast_D_leave                        (GtkButton       *button,
                                        gpointer         user_data)
{

}


void
on_tast_E_leave                        (GtkButton       *button,
                                        gpointer         user_data)
{

}


void
on_tast_F_leave                        (GtkButton       *button,
                                        gpointer         user_data)
{

}


void
on_tast_G_leave                        (GtkButton       *button,
                                        gpointer         user_data)
{

}


void
on_tast_H_leave                        (GtkButton       *button,
                                        gpointer         user_data)
{

}


void
on_tast_I_leave                        (GtkButton       *button,
                                        gpointer         user_data)
{

}


void
on_tast_J_leave                        (GtkButton       *button,
                                        gpointer         user_data)
{

}


void
on_tast_K_leave                        (GtkButton       *button,
                                        gpointer         user_data)
{

}


void
on_tast_L_leave                        (GtkButton       *button,
                                        gpointer         user_data)
{

}


void
on_tast_M_leave                        (GtkButton       *button,
                                        gpointer         user_data)
{

}


void
on_tast_N_leave                        (GtkButton       *button,
                                        gpointer         user_data)
{

}


void
on_tast_O_leave                        (GtkButton       *button,
                                        gpointer         user_data)
{

}


void
on_tast_P_leave                        (GtkButton       *button,
                                        gpointer         user_data)
{

}


void
on_tast_Q_leave                        (GtkButton       *button,
                                        gpointer         user_data)
{

}


void
on_tast_R_leave                        (GtkButton       *button,
                                        gpointer         user_data)
{

}


void
on_tast_S_leave                        (GtkButton       *button,
                                        gpointer         user_data)
{

}


void
on_tast_T_leave                        (GtkButton       *button,
                                        gpointer         user_data)
{

}


void
on_tast_U_leave                        (GtkButton       *button,
                                        gpointer         user_data)
{

}


void
on_tast_V_leave                        (GtkButton       *button,
                                        gpointer         user_data)
{

}


void
on_tast_W_leave                        (GtkButton       *button,
                                        gpointer         user_data)
{

}


void
on_tast_X_leave                        (GtkButton       *button,
                                        gpointer         user_data)
{

}


void
on_tast_Y_leave                        (GtkButton       *button,
                                        gpointer         user_data)
{

}


void
on_tast_leer_leave                     (GtkButton       *button,
                                        gpointer         user_data)
{

}


void
on_tast_Z_leave                        (GtkButton       *button,
                                        gpointer         user_data)
{

}


void
on_tast_slash_leave                    (GtkButton       *button,
                                        gpointer         user_data)
{

}


void
on_tast_dot_leave                      (GtkButton       *button,
                                        gpointer         user_data)
{

}


void
on_tast_vor_leave                      (GtkButton       *button,
                                        gpointer         user_data)
{

}


void
on_tast_0_leave                        (GtkButton       *button,
                                        gpointer         user_data)
{

}


void
on_tast_1_leave                        (GtkButton       *button,
                                        gpointer         user_data)
{

}


void
on_tast_2_leave                        (GtkButton       *button,
                                        gpointer         user_data)
{

}


void
on_tast_3_leave                        (GtkButton       *button,
                                        gpointer         user_data)
{

}


void
on_tast_4_leave                        (GtkButton       *button,
                                        gpointer         user_data)
{

}


void
on_tast_5_leave                        (GtkButton       *button,
                                        gpointer         user_data)
{

}


void
on_tast_6_leave                        (GtkButton       *button,
                                        gpointer         user_data)
{

}


void
on_tast_7_leave                        (GtkButton       *button,
                                        gpointer         user_data)
{

}


void
on_tast_8_leave                        (GtkButton       *button,
                                        gpointer         user_data)
{

}


void
on_tast_9_leave                        (GtkButton       *button,
                                        gpointer         user_data)
{

}


void
on_tast_esc_leave                      (GtkButton       *button,
                                        gpointer         user_data)
{

}


void
on_tast_back_leave                     (GtkButton       *button,
                                        gpointer         user_data)
{

}


void
on_tast_a_leave                        (GtkButton       *button,
                                        gpointer         user_data)
{

}


void
on_tast_b_leave                        (GtkButton       *button,
                                        gpointer         user_data)
{

}


void
on_tast_c_leave                        (GtkButton       *button,
                                        gpointer         user_data)
{

}


void
on_tast_d_leave                        (GtkButton       *button,
                                        gpointer         user_data)
{

}


void
on_tast_e_leave                        (GtkButton       *button,
                                        gpointer         user_data)
{

}


void
on_tast_f_leave                        (GtkButton       *button,
                                        gpointer         user_data)
{

}


void
on_tast_g_leave                        (GtkButton       *button,
                                        gpointer         user_data)
{

}


void
on_tast_h_leave                        (GtkButton       *button,
                                        gpointer         user_data)
{

}


void
on_tast_i_leave                        (GtkButton       *button,
                                        gpointer         user_data)
{

}


void
on_tast_j_leave                        (GtkButton       *button,
                                        gpointer         user_data)
{

}


void
on_tast_k_leave                        (GtkButton       *button,
                                        gpointer         user_data)
{

}


void
on_tast_l_leave                        (GtkButton       *button,
                                        gpointer         user_data)
{

}


void
on_tast_m_leave                        (GtkButton       *button,
                                        gpointer         user_data)
{

}


void
on_tast_n_leave                        (GtkButton       *button,
                                        gpointer         user_data)
{

}


void
on_tast_o_leave                        (GtkButton       *button,
                                        gpointer         user_data)
{

}


void
on_tast_p_leave                        (GtkButton       *button,
                                        gpointer         user_data)
{

}


void
on_tast_q_leave                        (GtkButton       *button,
                                        gpointer         user_data)
{

}


void
on_tast_r_leave                        (GtkButton       *button,
                                        gpointer         user_data)
{

}


void
on_tast_s_leave                        (GtkButton       *button,
                                        gpointer         user_data)
{

}


void
on_tast_t_leave                        (GtkButton       *button,
                                        gpointer         user_data)
{

}


void
on_tast_u_leave                        (GtkButton       *button,
                                        gpointer         user_data)
{

}


void
on_tast_v_leave                        (GtkButton       *button,
                                        gpointer         user_data)
{

}


void
on_tast_w_leave                        (GtkButton       *button,
                                        gpointer         user_data)
{

}


void
on_tast_x_leave                        (GtkButton       *button,
                                        gpointer         user_data)
{

}


void
on_tast_y_leave                        (GtkButton       *button,
                                        gpointer         user_data)
{

}


void
on_tast_leer1_leave                    (GtkButton       *button,
                                        gpointer         user_data)
{

}


void
on_tast_z_leave                        (GtkButton       *button,
                                        gpointer         user_data)
{

}


void
on_tast_slash1_leave                   (GtkButton       *button,
                                        gpointer         user_data)
{

}


void
on_tast_dot1_leave                     (GtkButton       *button,
                                        gpointer         user_data)
{

}


void
on_tast_colon_leave                    (GtkButton       *button,
                                        gpointer         user_data)
{

}


void
on_tast_dash_leave                     (GtkButton       *button,
                                        gpointer         user_data)
{

}


void
on_tast_www_leave                      (GtkButton       *button,
                                        gpointer         user_data)
{

}


void
on_tast_de_leave                       (GtkButton       *button,
                                        gpointer         user_data)
{

}


void
on_tast_com_leave                      (GtkButton       *button,
                                        gpointer         user_data)
{

}


void
on_tast_org_leave                      (GtkButton       *button,
                                        gpointer         user_data)
{

}


void
on_tast_esc1_leave                     (GtkButton       *button,
                                        gpointer         user_data)
{

}


void
on_tast_back1_leave                    (GtkButton       *button,
                                        gpointer         user_data)
{

}


void
on_tast_enter_leave                    (GtkButton       *button,
					gpointer         user_data)
{
	
}


void
on_tast_enter1_leave                    (GtkButton       *button,
					gpointer         user_data)
{
		
}


gboolean
on_eventbox_URI_button_press_event     (GtkWidget       *widget,
                                        GdkEventButton  *event,
                                        gpointer         user_data)
{
	static gboolean open = FALSE;
	
	open = ~open;
	
	if (open) 
	{
		hide_tastatur (widget);
		gtk_widget_show (combo_list);
		gtk_window_move (combo_list, 320, 40);
	}
	else 
	{
		gtk_widget_hide (combo_list);
		g_signal_emit_by_name (lookup_widget (windowTop,
						      "combo_entry_URI_entry"),
				       "activate");
	}
	
	g_print ("%s\n", __FUNCTION__);
	return FALSE;
}


gboolean
on_combo_entry_URI_scroll_event        (GtkWidget       *widget,
                                        GdkEvent        *event,
                                        gpointer         user_data)
{
	g_print ("%s\n", __FUNCTION__);
	return FALSE;
}


gboolean
on_combo_entry_URI_entry_scroll_event  (GtkWidget       *widget,
                                        GdkEvent        *event,
                                        gpointer         user_data)
{
	g_print ("%s\n", __FUNCTION__);
	return FALSE;
}


void print_atk_dom (AtkObject *accessible)
{
	gint i;
	
	for (i = atk_object_get_n_accessible_children (accessible); i; i--)
	{
		print_atk_dom (atk_object_ref_accessible_child (accessible, 
								i - 1));
	}
		
	g_print (">%s\n", atk_object_get_name (accessible));
}

void
on_combo_list_show                     (GtkWidget       *widget,
                                        gpointer         user_data)
{
	gint	i;
	gchar	*data;
	
	g_print ("%s: #%d entries\n", __FUNCTION__, 
		 g_list_length (URI_history));
	activate_pengudrive (__FUNCTION__ ,g_list_length (URI_history) + 1, 1);
	URI_history_pos = 0;	
	for (i = 0; i < 10; i++)
	{
		data = g_list_nth_data (URI_history, i);
		if (data) 
		{
			g_print ("%02d: %s\n", i, data);
			gtk_label_set_text (URIlabel [i], data);
			gtk_widget_show (URIlabel [i]);
		}
		else gtk_widget_hide (URIlabel [i]);
	}
}


gboolean
on_eventbox_URI_list_scroll_event      (GtkWidget       *widget,
                                        GdkEvent        *event,
                                        gpointer         user_data)
{
	gchar *data;
	guint button = event->scroll.direction;
	
	g_print ("%s\n", __FUNCTION__);
	URI_scrolled = TRUE;
	gtk_label_select_region (URIlabel [URI_history_pos - 1], -1, -1);
	switch (button)
	{
	case GDK_SCROLL_UP:
		URI_history_pos--;
		break;
	case GDK_SCROLL_DOWN:
		URI_history_pos++;
		break;
	}
	
	if (URI_history_pos <= 0)
	{
		URI_history_pos = 0;
		gtk_entry_set_text (lookup_widget (windowTop, 
						   "combo_entry_URI_entry"), 
				    " ");
	}
	else
	{
		while (!g_list_nth_data (URI_history, URI_history_pos - 1))
		{ 
			if (URI_history_pos == 0) break;
			URI_history_pos--; 
		}
		
		data = g_list_nth_data (URI_history, URI_history_pos - 1);
		if (data)
		{
			gtk_label_select_region (URIlabel [URI_history_pos - 1], 0, -1);
			gtk_entry_set_text (lookup_widget (windowTop, 
							   "combo_entry_URI_entry"), 
					    data);
		}
	}
	
	activate_pengudrive (__FUNCTION__ ,g_list_length (URI_history) + 1, 
			     URI_history_pos);
	g_print ("URI_history_pos = %d\n", URI_history_pos);
	return FALSE;
}


gboolean
on_eventbox_URI_list_button_press_event
                                        (GtkWidget       *widget,
                                        GdkEventButton  *event,
                                        gpointer         user_data)
{
	gtk_widget_hide (combo_list);
	g_signal_emit_by_name (lookup_widget (windowTop,
					      "combo_entry_URI_entry"),
			       "activate");
	return FALSE;
}


void
on_combo_list_hide                     (GtkWidget       *widget,
                                        gpointer         user_data)
{
	disable_pengudrive (__FUNCTION__);
}


void
on_hbox4_hide                          (GtkWidget       *widget,
                                        gpointer         user_data)
{
//	disable_pengudrive (__FUNCTION__);
//	gtk_widget_hide (combo_list);
}


gboolean
on_labelURI1_button_press_event        (GtkWidget       *widget,
                                        GdkEventButton  *event,
                                        gpointer         user_data)
{
	gchar *data;
	
	g_print ("%s: pressed\n", __FUNCTION__);
	data = g_list_nth_data (URI_history, 0);
	        if (data)
	{
		gtk_label_select_region (URIlabel [0], 0, -1);
		gtk_entry_set_text (lookup_widget (windowTop,
						   "combo_entry_URI_entry"),
				    data);
	}
	
	gtk_widget_hide (combo_list);
	g_signal_emit_by_name (lookup_widget (windowTop,
					      "combo_entry_URI_entry"),
			       "activate");
	return FALSE;
}


gboolean
on_labelURI2_button_press_event        (GtkWidget       *widget,
                                        GdkEventButton  *event,
                                        gpointer         user_data)
{
	gchar *data;
	
	g_print ("%s: pressed\n", __FUNCTION__);
	data = g_list_nth_data (URI_history, 0);
	        if (data)
	{
		gtk_label_select_region (URIlabel [0], 0, -1);
		gtk_entry_set_text (lookup_widget (windowTop,
						   "combo_entry_URI_entry"),
				    data);
	}
	
	gtk_widget_hide (combo_list);
	g_signal_emit_by_name (lookup_widget (windowTop,
					      "combo_entry_URI_entry"),
			       "activate");
	return FALSE;
}


gboolean
on_labelURI3_button_press_event        (GtkWidget       *widget,
                                        GdkEventButton  *event,
                                        gpointer         user_data)
{
	gchar *data;
	
	g_print ("%s: pressed\n", __FUNCTION__);
	data = g_list_nth_data (URI_history, 0);
	        if (data)
	{
		gtk_label_select_region (URIlabel [0], 0, -1);
		gtk_entry_set_text (lookup_widget (windowTop,
						   "combo_entry_URI_entry"),
				    data);
	}
	
	gtk_widget_hide (combo_list);
	g_signal_emit_by_name (lookup_widget (windowTop,
					      "combo_entry_URI_entry"),
			       "activate");
	return FALSE;
}


gboolean
on_labelURI4_button_press_event        (GtkWidget       *widget,
                                        GdkEventButton  *event,
                                        gpointer         user_data)
{
	gchar *data;
	
	g_print ("%s: pressed\n", __FUNCTION__);
	data = g_list_nth_data (URI_history, 0);
	        if (data)
	{
		gtk_label_select_region (URIlabel [0], 0, -1);
		gtk_entry_set_text (lookup_widget (windowTop,
						   "combo_entry_URI_entry"),
				    data);
	}
	
	gtk_widget_hide (combo_list);
	g_signal_emit_by_name (lookup_widget (windowTop,
					      "combo_entry_URI_entry"),
			       "activate");
	return FALSE;
}


gboolean
on_labelURI5_button_press_event        (GtkWidget       *widget,
                                        GdkEventButton  *event,
                                        gpointer         user_data)
{
	gchar *data;
	
	g_print ("%s: pressed\n", __FUNCTION__);
	data = g_list_nth_data (URI_history, 0);
	        if (data)
	{
		gtk_label_select_region (URIlabel [0], 0, -1);
		gtk_entry_set_text (lookup_widget (windowTop,
						   "combo_entry_URI_entry"),
				    data);
	}
	
	gtk_widget_hide (combo_list);
	g_signal_emit_by_name (lookup_widget (windowTop,
					      "combo_entry_URI_entry"),
			       "activate");
	return FALSE;
}


gboolean
on_labelURI6_button_press_event        (GtkWidget       *widget,
                                        GdkEventButton  *event,
                                        gpointer         user_data)
{
	gchar *data;
	
	g_print ("%s: pressed\n", __FUNCTION__);
	data = g_list_nth_data (URI_history, 0);
	        if (data)
	{
		gtk_label_select_region (URIlabel [0], 0, -1);
		gtk_entry_set_text (lookup_widget (windowTop,
						   "combo_entry_URI_entry"),
				    data);
	}
	
	gtk_widget_hide (combo_list);
	g_signal_emit_by_name (lookup_widget (windowTop,
					      "combo_entry_URI_entry"),
			       "activate");
	return FALSE;
}


gboolean
on_labelURI7_button_press_event        (GtkWidget       *widget,
                                        GdkEventButton  *event,
                                        gpointer         user_data)
{
	gchar *data;
	
	g_print ("%s: pressed\n", __FUNCTION__);
	data = g_list_nth_data (URI_history, 0);
	        if (data)
	{
		gtk_label_select_region (URIlabel [0], 0, -1);
		gtk_entry_set_text (lookup_widget (windowTop,
						   "combo_entry_URI_entry"),
				    data);
	}
	
	gtk_widget_hide (combo_list);
	g_signal_emit_by_name (lookup_widget (windowTop,
					      "combo_entry_URI_entry"),
			       "activate");
	return FALSE;
}


gboolean
on_labelURI8_button_press_event        (GtkWidget       *widget,
                                        GdkEventButton  *event,
                                        gpointer         user_data)
{
	gchar *data;
	
	g_print ("%s: pressed\n", __FUNCTION__);
	data = g_list_nth_data (URI_history, 0);
	        if (data)
	{
		gtk_label_select_region (URIlabel [0], 0, -1);
		gtk_entry_set_text (lookup_widget (windowTop,
						   "combo_entry_URI_entry"),
				    data);
	}
	
	gtk_widget_hide (combo_list);
	g_signal_emit_by_name (lookup_widget (windowTop,
					      "combo_entry_URI_entry"),
			       "activate");
	return FALSE;
}


gboolean
on_labelURI9_button_press_event        (GtkWidget       *widget,
                                        GdkEventButton  *event,
                                        gpointer         user_data)
{
	gchar *data;
	
	g_print ("%s: pressed\n", __FUNCTION__);
	data = g_list_nth_data (URI_history, 0);
	        if (data)
	{
		gtk_label_select_region (URIlabel [0], 0, -1);
		gtk_entry_set_text (lookup_widget (windowTop,
						   "combo_entry_URI_entry"),
				    data);
	}
	
	gtk_widget_hide (combo_list);
	g_signal_emit_by_name (lookup_widget (windowTop,
					      "combo_entry_URI_entry"),
			       "activate");
	return FALSE;
}


gboolean
on_labelURI10_button_press_event       (GtkWidget       *widget,
                                        GdkEventButton  *event,
                                        gpointer         user_data)
{
	gchar *data;
	
	g_print ("%s: pressed\n", __FUNCTION__);
	data = g_list_nth_data (URI_history, 0);
	        if (data)
	{
		gtk_label_select_region (URIlabel [0], 0, -1);
		gtk_entry_set_text (lookup_widget (windowTop,
						   "combo_entry_URI_entry"),
				    data);
	}
	
	gtk_widget_hide (combo_list);
	g_signal_emit_by_name (lookup_widget (windowTop,
					      "combo_entry_URI_entry"),
			       "activate");
	return FALSE;
}


gboolean
on_arrow1_enter_notify_event           (GtkWidget       *widget,
                                        GdkEventCrossing *event,
                                        gpointer         user_data)
{
	g_print ("%s\n", __FUNCTION__);
	return FALSE;
}


gboolean
on_arrow1_leave_notify_event           (GtkWidget       *widget,
                                        GdkEventCrossing *event,
                                        gpointer         user_data)
{
	g_print ("%s\n", __FUNCTION__);
	return FALSE;
}


gboolean
on_eventbox_URI_enter_notify_event     (GtkWidget       *widget,
                                        GdkEventCrossing *event,
                                        gpointer         user_data)
{
	g_print ("%s\n", __FUNCTION__);
	activate_pengudrive (__FUNCTION__ ,g_list_length (URI_history) + 1,
			     URI_history_pos);
	return FALSE;
}


gboolean
on_eventbox_URI_leave_notify_event     (GtkWidget       *widget,
                                        GdkEventCrossing *event,
                                        gpointer         user_data)
{
	g_print ("%s\n", __FUNCTION__);
	disable_pengudrive (__FUNCTION__);
	return FALSE;
}



gboolean
on_eventbox8_enter_notify_event        (GtkWidget       *widget,
                                        GdkEventCrossing *event,
                                        gpointer         user_data)
{
	g_print ("%s\n", __FUNCTION__);
	activate_pengudrive (__FUNCTION__ ,g_list_length (URI_history) + 1,
			     URI_history_pos);
	return FALSE;
}


gboolean
on_eventbox8_leave_notify_event        (GtkWidget       *widget,
                                        GdkEventCrossing *event,
                                        gpointer         user_data)
{
	g_print ("%s\n", __FUNCTION__);
	disable_pengudrive (__FUNCTION__);
	return FALSE;
}


gboolean
on_eventbox8_button_press_event        (GtkWidget       *widget,
                                        GdkEventButton  *event,
                                        gpointer         user_data)
{
	g_print ("%s\n", __FUNCTION__);
	
	if (!GTK_WIDGET_VISIBLE(combo_list))
	{
		gtk_widget_show (combo_list);
		gtk_window_move (combo_list, 320, 40);
	}
	else 
	{
		on_combo_entry_URI_entry_activate (lookup_widget (widget,
								  "combo_entry_URI_entry"),
						   NULL);
	}
	
	return FALSE;
}

